/**
 * There is one DataDirectorController.  If we were to have many DataDirector app instances
 * there would be one of these controllers per instance.
 * 
 * This is the master controller for the DataDirector app.  This controller is available
 * to all presentation models in the app. All remote messages are funneled through this class
 * before they go to the mid-tier and after the response comes back from the mid-tier.
 * 
 * If models need to be shared between non-related presentation models, those models
 * should be accessed through this controller.  Having one copy of a model that is shared
 * throughout the app allows components to be decoupled from one another which makes
 * mocking/testing individual pieces possible.  It also allows for easy sharing of data
 * between disparate views without having to resort to complicated eventing.
 */

define(["dojo/_base/declare",
        "dojo/_base/config",
        "dojo/Stateful",
        "dojo/Evented",
        "dojo/_base/lang",
        "dojo/request",
        "dojo/json",
        "dojo/has",
        "dojo/query",
        "dojo/string",
        "dojo/dom-class",
        "dijit/registry",
        "com/sas/dm/common/util/XhrUtils",
        "com/sas/dm/common/util/BindingUtils",
        "com/sas/dm/common/util/ArrayUtils",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/common/components/ErrorMessage",
		"com/sas/dm/common/components/ErrorDialog",
        "com/sas/dm/common/components/ConfirmationDialog",
        "com/sas/dm/common/preferences/PreferencesProxy",
        "com/sas/dm/common/preferences/PreferencesServiceStrategy",
        "com/sas/dm/datadirector/engine/DirectiveStepEngine",
        "com/sas/dm/datadirector/util/SharedProperties",
        "com/sas/dm/datadirector/util/StepHelper",
        "com/sas/dm/datadirector/util/DirectiveHelper",
        "com/sas/dm/datadirector/models/Step",
        "com/sas/dm/datadirector/models/Directive",
        "com/sas/dm/datadirector/models/PersistedDirective",
        "com/sas/dm/datadirector/models/UserCredential",
        "com/sas/dm/datadirector/models/InventoryInfo",
        "com/sas/dm/datadirector/models/qkb/QKBDefinition",
        "com/sas/dm/datadirector/models/qkb/QKBLocale",
        "com/sas/dm/datadirector/models/qkb/QKBToken",
        "com/sas/dm/datadirector/models/servers/HadoopServer",
        "com/sas/dm/datadirector/models/servers/LASRServer",
        "com/sas/dm/datadirector/models/servers/JdbcServer",
        "com/sas/dm/datadirector/models/servers/ImpalaServer",
        "com/sas/dm/datadirector/models/request/DataSampleRequest",
        "com/sas/dm/datadirector/models/ExecutionEnvironment",
        "com/sas/dm/datadirector/observer/DirectiveObserver",
        "com/sas/dm/common/model/factory/DeserializedDateFactory",
        "com/sas/dm/common/model/ErrorResponse",
        "com/sas/dm/datadirector/views/components/ConfigurationDialog",
        "com/sas/dm/datadirector/views/components/UpdateConfigurationDialog",
        "com/sas/dm/datadirector/views/components/ManageSavedDirectivesDialog",
        "com/sas/dm/datadirector/widgets/Toast",
        "dojox/uuid/generateRandomUuid",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], 
    function(declare, config, Stateful, Evented, _lang, request, Json, has, query, dojoString, domClass, registry, XhrUtils, BindingUtils, ArrayUtils, StringUtils, ErrorMessage, ErrorDialog, ConfirmationDialog, PreferencesProxy, PreferencesServiceStrategy, DirectiveStepEngine, SharedProperties, StepHelper, DirectiveHelper, Step, Directive, PersistedDirective, UserCredential, InventoryInfo, QKBDefinition, QKBLocale, QKBToken, HadoopServer, LASRServer, JdbcServer, ImpalaServer, DataSampleRequest, ExecutionEnvironment, DirectiveObserver, DateFactory, ErrorResponse, ConfigurationDialog, UpdateConfigurationDialog, ManageSavedDirectivesDialog, Toast, generateUuid, resourceBundle){
		var DataDirectorController = declare([Stateful, Evented],{
			
			// a list of all the steps available to the app.
			_allSteps:null,
			// a map of all the steps available to the app by type
			_allStepsByType:null,
			// a generic object just for holding information that should only be loaded once per
			// controller instance - the function metadata will be stored here, for example.
			_genericCache:null,
			
			_directiveStepEngine:null,
			_directiveStepEngineStepWatcher:null,
            _dialogStartPathWatcher:null,

			_directiveObserver:null,
			_currentDirectivePath:null,
			
			_hiveServerConfigured:false,
			_hiveServerConfiguredGetter:function(){
				return this._hiveServerConfigured;
			},
			_hiveServerConfiguredSetter:function(value/*hiveServerConfigured*/){				
				this._hiveServerConfigured = value;
			},
			
			_currentStep:null,
			_currentStepGetter:function(){
				return this._currentStep;
			},
			_currentStepSetter:function(value/*Step*/){
				
				this._currentStep = value;
			},
			
			_currentDirective:null,
			_currentDirectiveGetter:function(){
				return this._currentDirective;
			},
			_currentDirectiveSetter:function(value/*Directive*/){
				if(value != null && value.get("$isTemplate")){
					this._currentDirective = this.showDirective(value);
				}else{
					this._currentDirective = value;
				}
				if(this._directiveObserver != null){
					this._directiveObserver.set("directive", value);
				}
				if (value == null) {
				    this._currentDirectivePath = null;
                }
			},
			
			_currentView:null,
			_currentViewGetter:function() {
				return this._currentView;
			},
			_currentViewSetter:function(value/*DirectiveView*/) {
				this._currentView = value;
			},
			
			_isCurrentDirectiveUnsaved:false,
			_isCurrentDirectiveUnsavedGetter:function(){
				return this._isCurrentDirectiveUnsaved;
			},
			_isCurrentDirectiveUnsavedSetter:function(value){
				this._isCurrentDirectiveUnsaved=value;
			},
			
			_userName:null,
			_userNameGetter:function() {
				return this._userName;
			},
			_userNameSetter:function(value/*String*/) {
				this._userName = value;
			},

			_sasFoldersHomeDirectoryGetter:function(){
				return config.sas.commons.application.userDetails.SASFoldersHomeDirectory;
			},

			_workspaceServerHomeDirectoryGetter:function(){
				return config.sas.commons.application.userDetails.WSFoldersHomeDirectory;
			},

			_grantedCapabilities:null,
			_grantedCapabilitiesGetter:function() {
				return this._grantedCapabilities;
			},
			_grantedCapabilitiesSetter:function(value/*Map*/) {
				this._grantedCapabilities = value;
			},
			
			_preferencesProxy:null,
			_preferencesProxySetter:function(value/*ignored*/){
				// intentionally blank
			},
			_preferencesProxyGetter:function(){
				return this._preferencesProxy;
			},
			
			showDirective:function(directive /*Directive*/){
				if(directive == null){
					return;
				}
				if(directive.get("type") == Directive.TYPE_EXTERNAL_APPLICATION){
					return null;
				}
				if(this._directiveStepEngine == null){
					this._directiveStepEngine = new DirectiveStepEngine();
				}else{
					BindingUtils.Unbind(this._directiveStepEngineStepWatcher);
				}
				if(this._directiveStepEngine){
					this._directiveStepEngine.on("unSavedDirective", _lang.hitch(this, this._onUnSavedDirective));
				}
				this._directiveStepEngineStepWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onCurrentStepChange), this._directiveStepEngine, "currentStep");
				this._directiveStepEngine.set("directive", directive);
				return directive;
			},
			
			loadDirectiveContents:function(/*String*/ directiveId, /*Function*/successHandler, /*Function*/errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "directives/persisted/"+directiveId, undefined, urlRoot, successHandler, errorHandler);
			},

			loadDirectiveTemplateContents:function(/*String*/ templateId, /*Function*/successHandler, /*Function*/errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "directives/templates/"+templateId, undefined, urlRoot, successHandler, errorHandler);
			},

			loadExecutionDirectiveContents:function(/*String*/ directiveId, /*Function*/successHandler, /*Function*/errorHandler){
			    var urlRoot = SharedProperties.defaultRootURL;
			    this._quickAsyncCallAsJSON("GET", "executions/"+directiveId+"/directive", undefined, urlRoot, successHandler, errorHandler);
			},
			
			openDirective:function(/*Directive*/ directive, /*String*/ directivePath){
				this.set("isCurrentDirectiveUnsaved",false);
				if(directive == null){
					return;
				}
				directive.set("$isTemplate", false);
				if (directivePath != null)  {
					directive.set("$requireFolderPathPrompt", false);
				} else {
					// no folder path - FORCE A PROMPT (i.e. show Save Dialog to select path)
					directive.set("$requireFolderPathPrompt", true);
				}
				if(this._directiveStepEngine == null){
					this._directiveStepEngine = new DirectiveStepEngine();
				}else{
					BindingUtils.Unbind(this._directiveStepEngineStepWatcher);
				}
				if(this._directiveStepEngine){
					this._directiveStepEngine.on("unSavedDirective", _lang.hitch(this, this._onUnSavedDirective));
				}
				this._directiveStepEngineStepWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onCurrentStepChange), this._directiveStepEngine, "currentStep");
				this._directiveStepEngine.load(directive);
				this.set("currentDirective", directive);
                this._currentDirectivePath = directivePath;
			},
			
			_onUnSavedDirective:function(e){
				this.set("isCurrentDirectiveUnsaved",true);
			},
			
			
			// TODO: change name of this method to loadDirectiveTemplates.
			loadSteps:function(/*Function*/successHandler, /*Function*/errorHandler, /* Class of expected return type */returnTypeClass){
				// TODO: load steps of given type from mid-tier registry
				// hard code in the ChooseDirective step for now;
				this._allSteps = new Array();
				this._allStepsByType = new Object();
				
				this._quickAsyncGet("directives", {},_lang.hitch(this, function(result){
						console.log("Directive list is back!"); 
						this._allSteps = result; 
						var allStepsLength = this._allSteps.length;
						for(var i=0; i<allStepsLength; i++){
							var step = this._allSteps[i];
							step.set("$isTemplate", true);
							var stepType = step.get("type");
							var byTypeArray = this._allStepsByType[stepType]; 
							if(byTypeArray == null){
								byTypeArray = this._allStepsByType[stepType] = new Array();
							}
							byTypeArray.push(step);
						}
						successHandler(this._allSteps);
				//	}));
				}), errorHandler, returnTypeClass);
				
			},
			
			loadStepDataProvider:function(/*Object*/ endpointDescriptor,/*Function*/ successHandler, /*Function*/ errorHandler, /*object*/ fileToPost){
				//TODO: remove this when finally loading transformations from mid-tier
				if(endpointDescriptor.get("uri") == "transforms"){
					var self = this;
				//	setTimeout(function(){
						if (endpointDescriptor.get("params") === "DM_CLENSEDATAINHADOOP"){
							successHandler(self._mockClenseTransformations);
						} else {
							successHandler(self._mockTransformations);
						}
				//	}, 100);
					return;
				} 
				/*// if we are loading a Hadoop tables metadata (ie columns), we add it to recently viewed list
				else if (endpointDescriptor.get("uri") == "data/hadoop/metadata"){
					var table = endpointDescriptor.get("params");
					if (table) {
						table = [table];
					}
					this._updateRecentlyViewedHadoopTables(table); 				
				}
				else if (endpointDescriptor.get("uri") == "data/hadoop/metadatas"){
					this._updateRecentlyViewedHadoopTables(endpointDescriptor.get("params"));				
				}*/
				
				//TODO: remove this when loading JDBC connections from mid-tier
				/*if(endpointDescriptor.get("uri") == "jdbc"){
					var self = this;
					if (!endpointDescriptor.get("params")){
						successHandler(self._mockJDBCConnections);
					} else {
						if (endpointDescriptor.get("params").get("name") === "Customer Information"){												
							successHandler(self._mockJDBCConnections2);
						} else if (endpointDescriptor.get("params").get("name") === "Sales Associates"){
							successHandler(self._mockJDBCConnections3);
						} else if (endpointDescriptor.get("params").get("name") === "catalog 1"){
							successHandler(self._mockJDBCConnections5);
						} else {
							successHandler(self._mockJDBCConnections4);
						}
					}
					return;
				}*/
				
			    var urlRoot = (endpointDescriptor.get("root")) ? endpointDescriptor.get("root") : SharedProperties.defaultRootURL;
				//if POSTing with a payload call the appropriate function
				if(fileToPost){
					this._quickAsyncCallAsMultipartFormData("POST", endpointDescriptor.get("uri"), fileToPost, urlRoot, successHandler, errorHandler, endpointDescriptor.get("returnValueType"));
				}
				else{
					this._quickAsyncCallAsJSON(endpointDescriptor.get("method"), endpointDescriptor.get("uri"), endpointDescriptor.get("params"), urlRoot, successHandler, errorHandler, endpointDescriptor.get("returnValueType"));//, endpointDescriptor.get("returnValueType"));
				}
			},
			
			goToStep:function(/*Step*/ step){
				if(this._directiveStepEngine != null){
					this._directiveStepEngine.goToStep(step, true);
				}
			},
			
			deleteTable:function(/*Table*/ table, successHandler,errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				var method = "data/table";
				this._quickAsyncDelete(method, table, successHandler, errorHandler, String);
			},
			
			notifyDeleteTable:function(/*Table*/ table){
				this.emit(SharedProperties.deleteTables, {"table":table});
			},

			notifyDeleteFile:function(/*Object*/ file){
				this.emit(SharedProperties.deleteFile, file);
			},
			
			launchTableViewer:function(/*String*/ resourcePath){
				var tableViewerURL = SharedProperties.defaultTableViewerURL; 
				if(resourcePath != null){
					var params = dojoString.substitute(SharedProperties.tableViewerParamFormat, [encodeURIComponent(resourcePath)])
					tableViewerURL = tableViewerURL+params;
				}
				this.openURLInNewTab(tableViewerURL, SharedProperties.defaultRootURL);
			},
		
			launchProfileViewer:function(/*int*/ reportId){
				var profileViewerURL = SharedProperties.profileViewerContextRoot;
				if(reportId != null){
					var params = dojoString.substitute(SharedProperties.profileReportViewerParamFormat, [reportId]);
					profileViewerURL = profileViewerURL+params;
				}
				this.openURLInNewTab(profileViewerURL, null);
			},
			
			_isRelativeURL:function(/*String*/url){
				var regex = /^(?:[a-z]+:)?\/\//i;
				return !regex.test(url);
			},
			
			openURLInNewTab:function(/*String*/ url, /*String*/rootURL){
				var urlToLaunch = url;
				//if(url.substr(0, 7) != "http://" || url.substr(0, 8) != "https://"){
				if(this._isRelativeURL(url)){
					// relative
					if(url.charAt(0) != "/"){
						urlToLaunch = this._buildURL((rootURL || SharedProperties.contextRoot), url);
					}
				}
				window.open(urlToLaunch);
			},
			
			openFileInNewTab:function(/*String*/url){
				this._openFileInNewTabInternal(url, true);
			},
			
			_openFileInNewTabInternal:function(fileURL/*String*/, checkForRelativeURL/*Boolean*/) {
				if(fileURL == null){
					return;
				}
				
				// in some browsers, this triggers the leaving the app warning, which it shouldn't
				// so we disable the warning handler and renable it after the call
				var originalFunc = window.onbeforeunload;
				window.onbeforeunload = null;
				
				var urlToLaunch = fileURL;
				if (checkForRelativeURL == true) {
					//if(fileURL.substr(0, 7) != "http://" || fileURL.substr(0, 8) != "https://"){
					if(this._isRelativeURL(fileURL)){
						// relative
						urlToLaunch = this._buildURL(SharedProperties.defaultRootURL, fileURL);
					}
				}
				if(has("ie") || has("trident")){
					window.open(urlToLaunch, "_blank");
				}else{
					window.location.assign(urlToLaunch);
				}
				
				// renable the leaving app warning handler
				setTimeout(function(){ window.onbeforeunload = originalFunc; }, 0);
			},
			
			_onCurrentStepChange:function(step /*Step*/){
				this.set("currentStep", step);
			},
									
			backToMain:function(){
				if(this.get("isCurrentDirectiveUnsaved")){
					 this._showUnsavedConfirmDialog();
				}else{
					this.set("currentDirective", null);
					 // clear the steps 
					 this._directiveStepEngine.clearStepProgress();
				}
			},
			_showUnsavedConfirmDialog:function(){
				confirmDialog = new ConfirmationDialog({
			        title: resourceBundle.unSavedDirectiveDialog_title,
			        content: resourceBundle.unSavedDirectiveDialog_content,
				 });
				 confirmDialog.show();
				 confirmDialog.set("closable", false);
				 confirmDialog.set("submitButtonHandler", _lang.hitch(this, function(){
					 this.set("isCurrentDirectiveUnsaved",false);
					 confirmDialog.destroyRecursive();
					 this.set("currentDirective", null);
					 
					 // clear the steps 
					 this._directiveStepEngine.clearStepProgress();
				 }));
				 confirmDialog.set("cancelButtonHandler", _lang.hitch(this, function(){
					 confirmDialog.destroyRecursive();
				 }));
			},
			
			activateStep:function(step /*models.Step*/){
				this._directiveStepEngine.goToStep(step);
			},
			
			removeStep:function(step /*models.Step*/){
				this._directiveStepEngine.removeStep(step);
			},
			
			loadDirectiveTemplateFromFullModel:function(/*Directive*/ directive, successHandler, errorHandler){
				this._quickAsyncPost("echo", directive, successHandler, errorHandler);
			},
			
			// this should trigger a generate sample
			refreshSubViews:function(){
				this.emit(SharedProperties.refreshSubViews);
			},
			
			closeSubViews:function(){
				this.emit(SharedProperties.closeSubViews);
			},
			
			scrollToStep:function(step){
				if(step != null){
					this.emit(SharedProperties.scrollToStep, {"step":step});
				}
			},
			
            generateCodePreview:function(/*Step*/ step, /*boolean*/ forceRefresh, /*Function*/ successHandler, /*Function*/ errorHandler){
                var directive = this.get("currentDirective");
                if(this._directiveStepEngine != null && step != null){
                    // make sure the outputs from the current step are written to the directive outputs
                    this._directiveStepEngine.writeStepOutputsToDirectiveOutput(step);
                }
                //writeStepOutputsToDirectiveOutput emits an event that causes isCurrentDirectiveUnsaved to be set to true 
                //and therefore the confirmation dialog will get displayed - setting the flag to false to prevent the dialog 
                this.set("isCurrentDirectiveUnsaved",false);
                
                var cloneDirective = DirectiveHelper.ShallowCloneFromLastStep(directive, step);
                cloneDirective.set("converterBean", "hiveCodePreviewDirectiveConverter");
                cloneDirective.set("id", generateUuid());
                this._quickAsyncPost("preview", cloneDirective, successHandler, errorHandler);
            },
			
			generateSample:function(/*Step*/ step, /*boolean*/ forceRefresh, /*Function*/ successHandler, /*Function*/ errorHandler){
				var directive = this.get("currentDirective");
				if(this._directiveStepEngine != null && step != null){
					// make sure the outputs from the current step are written to the directive outputs
					this._directiveStepEngine.writeStepOutputsToDirectiveOutput(step);
				}
				//writeStepOutputsToDirectiveOutput emits an event that causes isCurrentDirectiveUnsaved to be set to true 
				//and therefore the confirmation dialog will get displayed - setting the flag to false to prevent the dialog 
				this.set("isCurrentDirectiveUnsaved",false);
				
				var sampleDirective = DirectiveHelper.ShallowCloneFromLastStep(directive, step);
				sampleDirective.set("converterBean", "hiveDataSampleDirectiveConverter");
				sampleDirective.set("id", generateUuid());
				var sampleDirectiveProperties = sampleDirective.get("properties");
				var inputTable = DirectiveHelper.FindFirstInputTable(sampleDirective);
				var sampleRequest = new DataSampleRequest({"directive":sampleDirective, "inputTable":inputTable, "forceRefresh":forceRefresh});
				var onAfterPreferredExecutionEnvironment = _lang.hitch(this,function(executionEnvironment /*String*/){
					sampleDirectiveProperties.set(SharedProperties.directiveProperty_PreferredExecutionEnvironment, executionEnvironment);
					this._quickAsyncPost("sample", sampleRequest, successHandler, errorHandler);
				});
				if(sampleDirectiveProperties != null){
					// if properties is null, all bets are off
					sampleDirectiveProperties.set(SharedProperties.directiveProperty_isSampleDirective, "true");
					DirectiveHelper.GetPreferredExecutionEnvironment(directive, this, onAfterPreferredExecutionEnvironment);
				}else{
					this._quickAsyncPost("sample", sampleRequest, successHandler, errorHandler);
				}
			},
			
			commitDefinedOutputs:function(step /*Step*/, specificOutputs /*Optional, Array of keys to be committed*/){
				var outputsMap = null;
				if(specificOutputs != null){
					outputsMap = ArrayUtils.ArrayToMap(specificOutputs);
				}
				this._directiveStepEngine.writeStepOutputsToDirectiveOutput(step, outputsMap);
			},
			
			generateModelAttributes: function(step, /*Function*/ successHandler, /*Function*/ errorHandler) {
				var directive = this.get("currentDirective");
				this._directiveStepEngine.writeStepOutputsToDirectiveOutput(step);
				var lastNonBranchSelection = DirectiveHelper.FindFirstInputTable(directive);
				if (lastNonBranchSelection) {
					path = lastNonBranchSelection.get("fullName");
				} else {
					console.warn("could not find path attribute");
					return;
				}
				
				this.loadModelAttributes(path, successHandler, errorHandler);
			},

			loadModelAttributes:function(/*String*/ path, /*Function*/ successHandler, /*Function*/ errorHandler) {
				
			    var urlRoot = SharedProperties.defaultRootURL;
			    var uri = "parsescoring?&path="+path;
			    var returnValueType = "com/sas/dm/datadirector/models/scoring/ScoringModel";
				this._quickAsyncCallAsJSON("GET", uri, null, urlRoot, successHandler, errorHandler, returnValueType);
			},
			
			executeDirective:function(/*model.Directive*/directive, /*Function*/successHandler, /*Function*/errorHandler){
				//var serializedDirective = directive.serialize();
				//console.log("execute current directive from controller..."+serializedDirective);
				var oldId = null;
				if(directive != null){
					oldId = directive.get("id");
					oldIsTemplate = directive.get("$isTemplate");
					if(directive.get("$isTemplate") == true){
						// we're executing an unsaved template, need to generate a new id...
						directive.set("id", generateUuid());
					}
				}
				this._quickAsyncPost("execute", directive, successHandler, errorHandler);
				if(oldId != null && directive != null){
					if(directive.get("id") != oldId){
						directive.set("id", oldId);
					}
				}
			},
			
			queryExectionProgress:function(/*int*/executionId, /*Function*/ successHandler, /*Function*/ errorHandler){
				console.log("querying execution progress for exection id:"+executionId);
				this._quickAsyncGet("executions/"+executionId,undefined,successHandler, errorHandler);
			},
			
			// successHandler is a function with signature function(/*Date*/ result)
			// if the result date has a getTime() that returns 0, it means the date was unable
			// to be located or parsed
			// errorHandler is a function with signature function(/*ErrorResponse*/ error)
			queryBuildDate:function(/*Function*/ successHandler, /*Function*/ errorHandler){
				this._quickAsyncGet("info/builddate",undefined, successHandler, errorHandler);
			},
			
			queryFunctionList:function(/*String*/dialect,/*Function*/ successHandler, /*Function*/ errorHandler){
				if(dialect != null){
					if(this._genericCache[SharedProperties.genericCacheKey_functionList] != null && this._genericCache[SharedProperties.genericCacheKey_functionList][dialect] != null){
						successHandler(this._genericCache[SharedProperties.genericCacheKey_functionList][dialect]);
						return;
					}
					this._quickAsyncGet(dojoString.substitute("functions/${0}/metadata", [dialect]), null,
							_lang.hitch(this, function(/*Array of FunctionMetadata*/result){
								if(result != null){
									if(this._genericCache[SharedProperties.genericCacheKey_functionList] == null){
										this._genericCache[SharedProperties.genericCacheKey_functionList] = {};
									}
									this._genericCache[SharedProperties.genericCacheKey_functionList][dialect] = result;
									successHandler(result);
								}
							}), errorHandler);
				}else{
					successHandler(null);
				}
			},
			
			saveCurrentDirective:function(successHandler, errorHandler){
				var currentStep = this.get("currentStep");
				if(this._directiveStepEngine != null && currentStep != null){
					// make sure the outputs from the current step are written to the directive outputs
					this._directiveStepEngine.writeStepOutputsToDirectiveOutput(currentStep);
				}
				//writeStepOutputsToDirectiveOutput emits an event that causes isCurrentDirectiveUnsaved to be set to true 
				//and therefore the confirmation dialog will get displayed - setting the flag to false to prevent the dialog 
//				this.set("isCurrentDirectiveUnsaved",false);
				
				// If this directive has already been saved at least once, then just overwrite
				// it without prompting the user.
				var self = this;
				if((this._currentDirective.get("$isTemplate") == false) &&
					(this._currentDirective.get("$requireFolderPathPrompt") != true)){
					this.savedDirectiveExists(this._currentDirective.get("id"), function(result) {
						if (result === true) {
							// Save the current directive without prompting
							var persistedDirective = self._convertToPersistedDirective(self._currentDirective);
                            persistedDirective.set("folderPath", self._currentDirectivePath);
							self.executeSavePersisted(persistedDirective, function(result2) {
								if (self._currentView != null) {
									var toast = new Toast();
									toast.setMessage(resourceBundle.saveDirective_toast_message);
									self._currentView.addChild(toast);
									toast.show();
								}
								if (successHandler != null) {
									successHandler(result2);
								}
								self.set("isCurrentDirectiveUnsaved",false);
							},
							function(error2) {
								if (self._currentView != null) {
									self.displayError(error2);
								}
								if (errorHandler != null) {
									errorHandler(error2);
								}
							});
						}
						else {
							// Not saved yet, so prompt
							self._showSaveDialog(successHandler, errorHandler);
						}
					
					},
					function(error) {
						// Log the error to the console, but go ahead and prompt the user to save
						console.log("Unable to retrieve saved directives by name while saving the current directive");
						self._showSaveDialog(successHandler, errorHandler);
					});
				}else{
					this._showSaveDialog(successHandler, errorHandler);
				}
			},
			
			_saveDialogOkHandler:function(event){
				this.set("isCurrentDirectiveUnsaved",false);
			},
			
			_showSaveDialog:function(successHandler, errorHandler) {
				var nodeList = query(".mainApplicationContainer");
				if (nodeList != null && nodeList.length > 0) {
					var mainApp = registry.byNode(nodeList[0]);
					if (mainApp != null) {
						var dialog = new ManageSavedDirectivesDialog({region:"center", controller:this});
						mainApp.addChild(dialog);
						if (this._currentDirective != null) {
							dialog.setDirectiveName(this._currentDirective.get("name"));
							dialog.setDirectiveDescription(this._currentDirective.get("description"));
							dialog.setStartPath(this._currentDirectivePath);
							dialog.setOkHandler(_lang.hitch(this, this._saveDialogOkHandler));
						}
						dialog.show(mainApp, ManageSavedDirectivesDialog.MODE_SAVE);
					}
				}
			},
			
			showGenericConfirmationDialog:function(/*String*/title, /*String*/message){
				var confirmationDialog = new ConfirmationDialog({
					submitText: resourceBundle.OK,
					content: message,
					title: title
				});
				domClass.add(confirmationDialog.confirmDialogNode.domNode, "initializationErrorDialog");
				confirmationDialog.set("closable", false);
				confirmationDialog.set("cancelable", false);
				confirmationDialog.set("submitButtonHandler", _lang.hitch(this, function(){
					confirmationDialog.destroyRecursive();
				}));
				confirmationDialog.set("cancelButtonHandler", _lang.hitch(this, function(){
					confirmationDialog.destroyRecursive();
				}));
				confirmationDialog.show();
			},
			
			saveCurrentDirectiveAs:function(successHandler, errorHandler){
				var currentStep = this.get("currentStep");
				if(this._directiveStepEngine != null && currentStep != null){
					// make sure the outputs from the current step are written to the directive outputs
					this._directiveStepEngine.writeStepOutputsToDirectiveOutput(currentStep);
				}
				//writeStepOutputsToDirectiveOutput emits an event that causes isCurrentDirectiveUnsaved to be set to true 
				//and therefore the confirmation dialog will get displayed - setting the flag to false to prevent the dialog 
//				this.set("isCurrentDirectiveUnsaved",false);
				
				var nodeList = query(".mainApplicationContainer");
				if (nodeList != null && nodeList.length > 0) {
					var mainApp = registry.byNode(nodeList[0]);
					if (mainApp != null) {
                        if(this._dialogStartPathWatcher !==null){
                            BindingUtils.Unbind(this._dialogStartPathWatcher);
                        }
						var dialog = new ManageSavedDirectivesDialog({region:"center", controller:this});
                        this._dialogStartPathWatcher = BindingUtils.HandlePropertyChange(_lang.hitch(this, this._onDialogStartPathChange), dialog, "startPath");
						mainApp.addChild(dialog);
						if (this._currentDirective != null) {
							dialog.setDirectiveName(this._currentDirective.get("name"));
							dialog.setDirectiveDescription(this._currentDirective.get("description"));
							dialog.setStartPath(this._currentDirectivePath);
							dialog.setOkHandler(_lang.hitch(this, this._saveDialogOkHandler));
						}
						dialog.show(mainApp, ManageSavedDirectivesDialog.MODE_SAVE_AS);
					}
				}
			},
            _onDialogStartPathChange:function(value){
                this._currentDirectivePath=value;
            },
			manageSavedDirectives:function(successHandler/*Function*/) {
				var nodeList = query(".mainApplicationContainer");
				if (nodeList != null && nodeList.length > 0) {
					var mainApp = registry.byNode(nodeList[0]);
					if (mainApp != null) {
						var dialog = new ManageSavedDirectivesDialog({region:"center", controller:this});
						mainApp.addChild(dialog);
						dialog.setCloseHandler(successHandler);
						dialog.show(mainApp, ManageSavedDirectivesDialog.MODE_MANAGE);
					}
				}
			},
			
			//sends the current directive to the mid-tier to be saved
			executeSave:function(id/*String*/, name/*String*/, description/*String*/, folderPath/*String*/, successHandler, errorHandler) {
				//first convert the current directive to a PersistedDirective
				var oldId = this._currentDirective.get("id");
				var oldName = this._currentDirective.get("name");
				var oldDesc = this._currentDirective.get("description");
				var oldFolderPath = this._currentDirective.get("folderPath");
				var oldIsTemplate = this._currentDirective.get("$isTemplate");
				var oldRequirePrompt = this._currentDirective.get("$requireFolderPathPrompt");
				this._currentDirective.set("id", id);
				this._currentDirective.set("name", name);
				this._currentDirective.set("description", description);
				this._currentDirective.set("folderPath", folderPath);
				this._currentDirective.set("$isTemplate", false);
				this._currentDirective.set("$requireFolderPathPrompt", false);
				var persistedDirective = this._convertToPersistedDirective(this._currentDirective);
				
				// Now, set the ID, name, and description
				persistedDirective.set("id", id);
				persistedDirective.set("name", name);
				persistedDirective.set("description", description);
				persistedDirective.set("folderPath", folderPath);
				//make the POST call
				var self = this;
				this.executeSavePersisted(persistedDirective, successHandler,
				function(error) {
					
					// Reset the properties of the current directive
					self._currentDirective.set("id", oldId);
					self._currentDirective.set("name", oldName);
					self._currentDirective.set("description", oldDesc);
					self._currentDirective.set("folderPath", oldFolderPath);
					self._currentDirective.set("$isTemplate", oldIsTemplate);
					self._currentDirective.set("$requireFolderPathPrompt", oldRequirePrompt);
					if (errorHandler != null) {
						errorHandler(error);
					}
				});
			},
			
			// Sends a persisited directive to the mid-tier to be saved
			executeSavePersisted:function(persistedDirective/*PersistedDirective*/, successHandler, errorHandler) {
			    var self = this;
				var oldPath = this._currentDirectivePath;
                this._currentDirectivePath = persistedDirective.get("folderPath");
                var errHandler = function(error) {
                    self._currentDirectivePath = oldPath;
                    if (errorHandler != null) {
                        errorHandler(error);
                    }
                };
				//make the POST call
				this._quickAsyncPost("directives/persisted", persistedDirective, successHandler, errHandler, String);
			},
			
			//converts the directive into a PersistedDirective
			_convertToPersistedDirective:function(directive /*Directive*/) {
				var persisted = new PersistedDirective();
				
				//get the directive as a JSON string
				var jsonDirective = XhrUtils.getInstance().convertObjectsToJson(directive);
				
				persisted.set("directiveContent", jsonDirective);
				persisted.set("id", directive.get("id"));
				persisted.set("name", directive.get("name"));
				persisted.set("category", directive.get("category"));
				persisted.set("type", directive.get("type"));
				persisted.set("description", directive.get("description"));
				persisted.set("folderPath", directive.get("folderPath"));
				//persisted.set("owner", directive.get("owner"));
				persisted.set("isTemplate", false);
				persisted.set("iconClass", directive.get("iconClass"));
				persisted.set("createdDate", new Date()); //TODO:  what to do about dates here?
				persisted.set("modifiedDate", new Date());
				
				return persisted;
			},
			
			showConfigurationDialog:function() {
			    this.showConfigurationDialog(false);    
			},
			
			showConfigurationDialog:function(loadInventoryInfo /*boolean*/) {
				var nodeList = query(".mainApplicationContainer");
				if (nodeList != null && nodeList.length > 0) {
					var mainApp = registry.byNode(nodeList[0]);
					if (mainApp != null) {
						var dialog = new ConfigurationDialog({region:"center", controller:this, loadInventoryInfo:loadInventoryInfo});
						mainApp.addChild(dialog);
						dialog.show(mainApp);
					}
				}
			},

            showUpdateConfigurationDialog:function(inventoryInfo /*InventoryInfo*/, hadoopConfigPanePM /*HadoopConfigPanePM*/) {
                var nodeList = query(".mainApplicationContainer");
                if (nodeList != null && nodeList.length > 0) {
                    var mainApp = registry.byNode(nodeList[0]);
                    if (mainApp != null) {
                        var dialog = new UpdateConfigurationDialog({region:"center", controller:this, inventoryInfo:inventoryInfo, hadoopConfigPanePM:hadoopConfigPanePM});
                        mainApp.addChild(dialog);
                        dialog.show(mainApp);
                    }
                }    
            },
            
			loadProfileDirectories:function(successHandler, errorHandler){
				
				var root = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "directories/profile", null, root, successHandler, errorHandler, "com/sas/dm/datadirector/models/DirectoryInfo");
			},
			loadSavedDirectives:function(successHandler, errorHandler) {
				var root = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "directives/persisted", null, root, successHandler, errorHandler, "com/sas/dm/datadirector/models/PersistedDirectiveInfo");
			},
			
			loadChainedDirectiveInformation:function(/*String*/ directiveId, /*Function*/successHandler, /*Function*/errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "directives/chained/persisted/"+directiveId, undefined, urlRoot, successHandler, errorHandler);
			},
			
			getSavedDirective:function(id, successHandler, errorHandler) {
				this._quickAsyncGet("directives/persisted/" + id, null, successHandler, errorHandler, "com/sas/dm/datadirector/models/Directive");
            },
			
			getSavedDirectivesByName:function(name, successHandler, errorHandler) {
				this._quickAsyncGet("directives/persisted?name=" + encodeURIComponent(name), null, successHandler, errorHandler, "com/sas/dm/datadirector/models/PersistedDirective");
            },
            
            deleteSavedDirectives:function(ids/*Array of Strings*/, successHandler, errorHandler) {
            	var method = "directives/persisted?";
            	for (var i = 0; i < ids.length; i++) {
            		if (i > 0) {
            			method += "&";
            		}
            		method += "ids=";
            		method += ids[i];
            	}
				this._quickAsyncDelete(method, null, successHandler, errorHandler, String);
            },
            
            deleteRunStatuses:function(ids/*Array of Strings*/, successHandler, errorHandler) {
            	var method = "executions?executionIds=";
            	for (var i = 0; i < ids.length; i++) {
            		method += ids[i];
            		if(i<ids.length-1){
            			method += ',';
            		}
            	}
            	console.log(method);
				this._quickAsyncDelete(method, null, successHandler, errorHandler, String);
            },
            
            renameSavedDirective:function(id/*String*/, name/*String*/, successHandler, errorHandler) {
            	var method = "directives/persisted/" + id + "/" + encodeURIComponent(name);
            	this._quickAsyncPost(method, null, successHandler, errorHandler, "com/sas/dm/datadirector/models/PersistedDirectiveInfo");
            },
            
            savedDirectiveExists:function(id/*String*/, successHandler, errorHandler) {
				this._quickAsyncGet("directives/persisted/exists/" + id, null, successHandler, errorHandler, null);  // Returns a primitive boolean, which XhrUtils will figure out on its own
            },

            moveSavedDirectives:function(ids/*Array of Strings*/, newParentFolder/*String*/, successHandler, errorHandler) {
                var uri = "directives/persisted?parent=" + encodeURIComponent(newParentFolder);
                this._quickAsyncPut(uri, ids, successHandler, errorHandler, String);
            },

			createProfileDirectory:function(directoryInfo, successHandler, errorHandler){
				var root = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", "directories", directoryInfo, root, successHandler, errorHandler, "com/sas/dm/datadirector/models/DirectoryInfo");
			},
			
			getProfileValuesFromColumn:function(column, successHandler, errorHandler) {
				var root = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", "/data/column/profile", column, root, successHandler, errorHandler, "com/sas/dm/datadirector/models/profile/ProfileColumnData");
			},

			getApplicationProperty:function(propertyName /*String*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("GET",
					"configuration/application/properties/" + encodeURIComponent(propertyName),
					null,
					SharedProperties.restRootURL,
					successHandler,
					errorHandler);
			},

			getCapabilities:function(successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("GET",
					"configuration/application/capabilities",
					null,
					SharedProperties.restRootURL,
					successHandler,
					errorHandler);
			},

            getQKBLocales:function(successHandler, errorHandler) {
                this._quickAsyncCallAsJSON("GET",
                    "configuration/qkb/locales",
                    null,
                    SharedProperties.restRootURL,
                    _lang.hitch(this, function(result){
                        if(result != null && result instanceof Array && result.length > 0){
                            successHandler(result);
                        }else{
                        	var message = resourceBundle.qkb_noLocaleDescription_txt;
                        	if(this.hasCapability(SharedProperties.capability_AdministerApplication)){
                        		message = resourceBundle.qkb_adminNoLocaleDescription_txt;
							}
                            var noLocalesError = new ErrorResponse({"message": resourceBundle.qkb_noLocaleError_txt,"details":message, "httpStatusCode":404});
                            errorHandler(noLocalesError);
                        }
                    }),
                    errorHandler,
                    QKBLocale);
            },

            getQKBDefinitions:function(locale /*QKBLocale*/, successHandler, errorHandler) {
                this._quickAsyncCallAsJSON("GET",
                    "configuration/qkb/definitions/?localeName=" + locale.get("name"),
                    null,
                    SharedProperties.restRootURL,
                    successHandler,
                    errorHandler,
                    QKBDefinition);
            },

            getQKBTokens:function(definition /*QKBDefinition*/, successHandler, errorHandler) {
                this._quickAsyncCallAsJSON("GET",
                    "configuration/qkb/tokens?localeName=" + definition.get("locale").get("name") +
                    "&definitionName=" + definition.get("name") +
                    "&definitionType=" + definition.get("type"),
                    null,
                    SharedProperties.restRootURL,
                    successHandler,
                    errorHandler,
                    QKBToken);
            },

            refreshQKBCache:function(successHandler, errorHandler) {
                this.getHadoopServer(
                    // Success
                    _lang.hitch(this, function(data /*JSON Object*/){
                        //this should always return exactly one server
                        if (data != null && data.length > 0) {
                            var server = data[0];
                            this._quickAsyncCallAsJSON("POST",
                                "configuration/server/"+server.get("id")+"/qkb/refresh?forceRefresh=true",
                                null,
                                SharedProperties.restRootURL,
                                successHandler,
                                _lang.hitch(this, function(error){
                                    if(error != null && error.get("httpStatusCode") == 404){
                                        error.set("message", "qkb_noLocaleError_txt");
                                    }
                                    errorHandler(error);
                                }),
                                HadoopServer);
                        } else {
                            successHandler();
                        } }),
                    // Error
                    errorHandler );
            },
	        
            getInventoryInfo:function(successHandler, errorHandler){
                this._quickAsyncGet("configuration/inventory", null, successHandler, errorHandler, InventoryInfo, SharedProperties.restRootURL);
	        },
	        	        	        	       	        	       
	        getJDBCConnections:function(successHandler, errorHandler){
				this._quickAsyncGet("configuration/server?type=JdbcServer", null, successHandler, errorHandler, JdbcServer, SharedProperties.restRootURL);
	        },
			
			setJDBCConnection:function(server /*JdbcServer*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("PUT", "configuration/server", server, SharedProperties.restRootURL, successHandler, errorHandler, JdbcServer);
			},
			
			deleteJDBCConnection:function(server /*JdbcServer*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("DELETE", "configuration/server/"+server.get("id"), null, SharedProperties.restRootURL, successHandler, errorHandler, String);
			},

			testConnection:function( path, successHandler, errorHandler) {
				var	method = "configuration/server/connect?path=" + path;
				this._quickAsyncPost(method, null, successHandler, errorHandler, String, SharedProperties.restRootURL);

			},

			createDirectoryBackup:function(successHandler, errorHandler) {
				var method = "dbbackup";
				this._quickAsyncPost(method, null, successHandler, errorHandler, String, SharedProperties.contextRoot+SharedProperties.uiMidTierURLPrefix);
			},
	            
			getHadoopServer:function(successHandler, errorHandler) {
				this._quickAsyncGet("configuration/server?type=HadoopServer", null, successHandler, errorHandler, HadoopServer, SharedProperties.restRootURL);
			},
			
			setHadoopServer:function(server /*HadoopServer*/, successHandler, errorHandler) {
			    // Hadoop Server updated -- QKB index should be retried
				this._quickAsyncCallAsJSON("PUT", "configuration/server", server, SharedProperties.restRootURL, successHandler, errorHandler, HadoopServer);
			},
			
			getImpalaServer:function(successHandler, errorHandler) {
				this._quickAsyncGet("configuration/server?type=ImpalaServer", null, successHandler, errorHandler, ImpalaServer, SharedProperties.restRootURL);
			},
			
			setImpalaServer:function(server /*HadoopServer*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("PUT", "configuration/server", server, SharedProperties.restRootURL, successHandler, errorHandler, ImpalaServer);
			},
			
			getLasrServers:function(successHandler, errorHandler) {
				this._quickAsyncGet("configuration/server?type=LASRServer", null, successHandler, errorHandler, LASRServer, SharedProperties.restRootURL);
			},
			
			setLasrServer:function(server /*LasrServer*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("PUT", "configuration/server", server, SharedProperties.restRootURL, successHandler, errorHandler, LASRServer);
			},
			
			deleteLasrServer:function(server /*LasrServer*/, successHandler, errorHandler) {
				this._quickAsyncCallAsJSON("DELETE", "configuration/server/"+server.get("id"), null, SharedProperties.restRootURL, successHandler, errorHandler, String);
			},
			
			getCredentials:function(domain, successHandler, errorHandler) {
				this._quickAsyncGet("configuration/server/credentials?domain=" + domain, null, successHandler, errorHandler, UserCredential, SharedProperties.restRootURL);
			},
			
			setCredentials:function(user, password, domain, successHandler, errorHandler) {
				var method = "configuration/server/credentials?user="+user+"&pass="+password+"&domain="+domain;
				this._quickAsyncCallAsJSON("PUT", method, null, SharedProperties.restRootURL, successHandler, errorHandler, null);
			},

			deleteCredentials:function(domain, successHandler, errorHandler) {
				this._quickAsyncDelete("configuration/server/credentials?domain=" + domain, null, successHandler, errorHandler, String, SharedProperties.restRootURL);
			},

	        downloadExternalFile:function(path) {
	        	var url = this._buildURL(SharedProperties.contextRoot + SharedProperties.uiMidTierURLPrefix, "datasource/externalDataFileStream?path=" + encodeURIComponent(path));
				this._openFileInNewTabInternal(url, false);
	        },

			uploadFile:function(file, path, overwrite, successHandler, errorHandler){
				var url = "file?path=" + encodeURIComponent(path);
				if(overwrite){
					url += "&overwrite="+overwrite;
				} else {
					url += "&overwrite=0";
				}
				this._quickAsyncCallAsMultipartFormData("POST", url, file, SharedProperties.defaultRootURL, successHandler, errorHandler, String);
			},
	        
	        getFileContents:function(path, successHandler, errorHandler) {
	        	var method = "externalDataFile?path="+encodeURIComponent(path);
	        	this._quickAsyncCallAsJSON("GET", method, null, SharedProperties.uiMidTierURLPrefix+"datasource", successHandler, errorHandler, null);
	        },

			deleteFile:function(/*String*/ path, successHandler,errorHandler){
				var urlRoot = SharedProperties.contextRoot + SharedProperties.uiMidTierURLPrefix;
				var method = "datasource/externalDataFile?path="+encodeURIComponent(path);
				this._quickAsyncDelete(method, null, successHandler, errorHandler, String, urlRoot);
			},

			getUserName:function() {
				this.set("userName", config.sas.commons.application.userDetails.userid);
			},

			// Check if user has been granted the specified capability
			hasCapability:function(capabilityName/*String*/) {
				if (this.get("grantedCapabilities") === null) {
					this.set("grantedCapabilities", ArrayUtils.BuildLookUpMap(config.sas.commons.application.grantedCapabilities, true));
				}

				var capsMap = this.get("grantedCapabilities");

				if (capabilityName === null) {
					return false;
				} else {
					var resolvedCap = capsMap[capabilityName];

					if (resolvedCap) {
						return true;
					} else {
						return false;
					}
				}
			},

            _executionEnvironmentCache:null,
            loadExecutionEnvironments:function(successHandler, errorHandler, preventListCaching /*Boolean, Optional (defaults to false) - true if list should not be cached*/ ){
                if(preventListCaching == true || this._executionEnvironmentCache == null){
                    var onSuccess = successHandler;
                    if(preventListCaching !== true){
                        onSuccess = _lang.hitch(this, function(result){
							this._executionEnvironmentCache = result;
							successHandler(result);
						});
                    }
                    this._loadExecutionEnvironments(onSuccess, errorHandler);
                }else{
                    successHandler(this._executionEnvironmentCache);
                }               
            },
           
            _loadExecutionEnvironments:function(successHandler, errorHandler){
				var uri = "availableExecutionEnvironments";
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", uri, null, urlRoot, successHandler, errorHandler, ExecutionEnvironment);
            },

            _sasWorkspaceServersCache:null,
            loadSASWorkspaceServers:function(successHandler, errorHandler, preventListCaching /*Boolean, Optional (defaults to false) - true if list should not be cached*/ ){
                if(preventListCaching == true || this._sasWorkspaceServersCache == null){
                    var onSuccess = successHandler;
                    if(preventListCaching !== true){
                        onSuccess = _lang.hitch(this, function(result){
                            this._sasWorkspaceServersCache= result;
                            successHandler(result);
                        });
                    }
                    this._loadSASWorkspaceServers(onSuccess, errorHandler);
                }else{
                    successHandler(this._sasWorkspaceServersCache);
                }
            },

            _loadSASWorkspaceServers:function(successHandler, errorHandler){
                var urlRoot = SharedProperties.defaultRootURL;
                this._quickAsyncCallAsJSON("GET", "workspaces", null, urlRoot, successHandler, errorHandler);
            },

			getUserPreference:function(name, successHandler, errorHandler) {
				this._preferencesProxy.get(name).then(
						_lang.hitch(this, function(result){
							successHandler({"key":name, "value":result});
						}), 
						_lang.hitch(this, errorHandler));
				//this._quickAsyncGet("users/" + this.get("userName") + "/preferences?name=" + name, null, successHandler, errorHandler, null, SharedProperties.restRootURL);
			},


            getRawUserPreference:function(name, successHandler, errorHandler) {
                //get the actual value from the preferences service, bypassing preferences cache/proxy so we know
                //if the value isn't coming from configuration.
				//TODO: In future (3.2) should fold this into preferences proxy, but too intrusive for defect
				//Introduced for S1315776 and used by HadoopConfigPanePM
                this._quickAsyncGet("users/"+this.get("userName")+"/preferences/raw?name="+name, null, successHandler, errorHandler, null, SharedProperties.restRootURL);
            },

			// use this if the user preference is a complex object. 
			// It will convert the string returned into the original class/object
			getComplexUserPreference:function(name, successHandler, errorHandler, classOfObject /*optional*/) {
				this._preferencesProxy.get(name).then(
						_lang.hitch(this, function(result){							
							XhrUtils.getInstance().convertJsonObjectsToClass(result, classOfObject).then( _lang.hitch(this, function(converted){
								result = converted;
								successHandler({"key":name, "value":result});
							}));							
						}), 
						_lang.hitch(this, errorHandler));
			},
			
			setUserPreference:function(name, value, successHandler, errorHandler) {
				this._preferencesProxy.set(name, value).then(_lang.hitch(this, successHandler), _lang.hitch(this, errorHandler));
//				var method = "users/" + this.get("userName") + "/preferences";
//				var prefobj = {"key": name, "value": value};
//				var params = {"preferences": [prefobj]};
//				this._quickAsyncCallAsJSON("PUT", method, params, SharedProperties.restRootURL, successHandler, errorHandler, null);
			},
			
			createHdfsDirectory:function(path, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "hdfsCreateDirectory?&path=" + encodeURIComponent(path);
			    var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", uri, null, urlRoot, successHandler, errorHandler);
			},

			createSasFolder:function(path, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "sasCreateFolder?&path=" + encodeURIComponent(path);
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", uri, null, urlRoot, successHandler, errorHandler);
			},

			deleteHdfsDirectory:function(path, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "hdfsDeleteDirectory?&path=" + encodeURIComponent(path);
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("DELETE", uri, null, urlRoot, successHandler, errorHandler);
			},

			deleteSasFolder:function(path, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "sasDeleteFolder?&path=" + encodeURIComponent(path);
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("DELETE", uri, null, urlRoot, successHandler, errorHandler);
			},

			deleteHdfsFile:function(path, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "hdfsDeleteFile?&path=" + encodeURIComponent(path);
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("DELETE", uri, null, urlRoot, successHandler, errorHandler);
			},
			
			renameHdfsDirectory:function(path, newName, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "hdfsRenameDirectory?&path=" + encodeURIComponent(path) +"&newName=" + encodeURIComponent(newName);
			    var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", uri, null, urlRoot, successHandler, errorHandler);
			},

			renameSasFolder:function(path, newName, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "sasRenameFolder?&path=" + encodeURIComponent(path) +"&newName=" + encodeURIComponent(newName);
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", uri, null, urlRoot, successHandler, errorHandler);
			},

			renameHdfsFile:function(path, newName, /*Function*/ successHandler, /*Function*/ errorHandler){
				var uri = "hdfsRenameFile?&path=" + encodeURIComponent(path) +"&newName=" + encodeURIComponent(newName);
			    var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("POST", uri, null, urlRoot, successHandler, errorHandler);
			},
			
			loadHdfsHomePathString:function(/*Function*/ successHandler, /*Function*/ errorHandler){
			    var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "hdfsRootDirectory", null, urlRoot, successHandler, errorHandler);
			},

			loadSasFoldersHomePathString:function(/*Function*/ successHandler, /*Function*/ errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "sasUserRoot", null, urlRoot, successHandler, errorHandler);
			},

			loadSasFoldersRootPathString:function(/*Function*/ successHandler, /*Function*/ errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
				this._quickAsyncCallAsJSON("GET", "sasRootFolder", null, urlRoot, successHandler, errorHandler);
			},

			loadHdfsPath: function(path /*String*/, successHandler, errorHandler, fileExtension){
				//fileExtension = (typeof fileExtension === 'undefined') ? 'ds2' : fileExtension;
			    var urlRoot = SharedProperties.defaultRootURL;
			    
			    var uri = (fileExtension) ? "hdfsChildrenByFileExtension?fileExtension=" + fileExtension + "&path=" + encodeURIComponent(path)
			    				: "hdfsChildren?path=" + encodeURIComponent(path);
			    var returnValueType = "com/sas/dm/datadirector/models/hierarchy/ContentFileInfo";
				this._quickAsyncCallAsJSON("GET", uri, null, urlRoot, successHandler, errorHandler, returnValueType);
			},

			loadSasFolderPath: function(path /*String*/, successHandler, errorHandler){
				this.loadSasFolderPathFilterByType(path, null, successHandler, errorHandler);
			},

			loadSasFolderPathFilterByType:function(path /*String*/, types /*Array of String*/, successHandler, errorHandler) {
				var urlRoot = SharedProperties.defaultRootURL;

				var uri = "sasFolderChildren?path=" + encodeURIComponent(path);
                if (types != null && types.length > 0) {
                    for (var i = 0; i < types.length; i++) {
                        uri = uri + "&types=" + encodeURIComponent(types[i]);
                    }
                }

				this._quickAsyncCallAsJSON("GET", uri, null, urlRoot, successHandler, errorHandler);
			},

			loadSasLibraryList:function(successHandler /*Function(SASLibrary[])*/, errorHandler, path /*String, optional */){
				var suffix = "/sasLibraries";
				if(StringUtils.isEmpty(path) == false){
					suffix += "?path="+encodeURIComponent(path);
				}
				this._quickAsyncGet(suffix, null, successHandler, errorHandler);
			},

			getProfileColumnMatch: function(profileMatchObj /*ProfileMatch*/, successHandler, errorHandler){
			    var urlRoot = SharedProperties.defaultRootURL;
			    var uri = "profileMatchColumns";			    
				this._quickAsyncCallAsJSON("POST", uri, profileMatchObj, urlRoot, successHandler, errorHandler);
			},
			
			getAllProfileColumnMatches: function(tables /*Array of TableMetadata*/, successHandler, errorHandler){
				var urlRoot = SharedProperties.defaultRootURL;
			    var uri = "profileValues";			    
				this._quickAsyncCallAsJSON("POST", uri, tables, urlRoot, successHandler, errorHandler);
			},						
			
			downloadColumnDefinitions:function(columns, location, successHandler, errorHandler){
				this._quickAsyncCallAsJSON("POST", "externalStructureFile/download", columns, SharedProperties.contextRoot+SharedProperties.uiMidTierURLPrefix+"datasource/", successHandler, errorHandler);
			},

			getLibname:function(libraryPath, successHandler, errorHandler){
				this._quickAsyncCallAsJSON("GET", "generateLibname?path="+libraryPath, undefined, SharedProperties.defaultRootURL, successHandler, errorHandler);
			},
			
	        _isAdvancedParameterInCurrentDirective:function(value /*String*/){     
                var directive = this.get("currentDirective");
                if(directive != null){
                    // these shenanigans are OK because the UI is single-threaded.                    
                    var advancedParameters = directive.get("advancedParams");
                    if(advancedParameters != null){
                        var advancedParametersLength = advancedParameters.length;
                        for(var i=0; i<advancedParametersLength; i++){
                            var advancedParam = advancedParameters[i];
                            if(advancedParam != null){
                                if(advancedParam.name == value){
                                    return true;
                                }
                            }
                        }
                    }
                }	            
	            
	            return false;
	        },
			
			_quickAsyncGet:function(methodName, params, successHandler, errorHandler, classOfReturnType /*, url */){
				var url = SharedProperties.defaultRootURL;
				if(arguments.length > 5){
					url = arguments[5];
				}
					
				this._quickAsyncCallAsJSON("GET", methodName, params, url, successHandler, errorHandler, classOfReturnType);
			},
			
			_quickAsyncPost:function(methodName, params, successHandler, errorHandler, classOfReturnType /*, url */){
				var url = SharedProperties.defaultRootURL;
				if(arguments.length > 5){
					url = arguments[5];
				}

				this._quickAsyncCallAsJSON("POST", methodName, params, url, successHandler, errorHandler, classOfReturnType);
			},
			
			_quickAsyncDelete:function(methodName, params, successHandler, errorHandler, classOfReturnType /*, url */){
				var url = SharedProperties.defaultRootURL;
				if(arguments.length > 5){
					url = arguments[5];
				}
					
				this._quickAsyncCallAsJSON("DELETE", methodName, params, url, successHandler, errorHandler, classOfReturnType);
			},

            _quickAsyncPut:function(methodName, params, successHandler, errorHandler, classOfReturnType /*, url */){
                var url = SharedProperties.defaultRootURL;
                if(arguments.length > 5){
                    url = arguments[5];
                }

                this._quickAsyncCallAsJSON("PUT", methodName, params, url, successHandler, errorHandler, classOfReturnType);
            },

			_quickAsyncCall:function(verb, methodName, params, url, successHandler, errorHandler, handleAs, classOfReturnType){
				var options = {};
				options.handleAs = handleAs;
				options.method = verb;
				options.preventCache = true;
				if(params !== undefined){
					options.data = XhrUtils.getInstance().convertObjectsToJson(params);
				}
//				if(typeof params != "string"){
//					options.data = Json.stringify(params);
//				}else{
//					options.data = params;
//				}
				options.headers = {"Accept": "application/json", "Content-Type":"application/json; charset=utf-8"};

				//multipart properties should be automatically handled so the boundary can be created
				if(handleAs === SharedProperties.restType_Multipart){
					options = {};
					options.handleAs = handleAs;
					options.method = verb;
					options.data = params;
					options.contentType = false;
					options.processData = false;
					options.headers = {"Accept": "application/json"};
				}
				request(this._buildURL(url, methodName), options).then(
					/*Success*/ _lang.hitch(this, function(result /*JSON Object*/){
						XhrUtils.getInstance().convertJsonObjectsToClass(result, classOfReturnType).then( _lang.hitch(this, function(converted){
							successHandler(converted);
						}));
					}), 
					/*Error*/_lang.hitch(this, function(error /*Object*/){
						if(error != null && error.response != null){
							var toConvert = error.response;
							if(handleAs === SharedProperties.restType_Multipart)
							{
								try{
									error.response.data= Json.parse(error.response.data);
								}catch(e){

								}
							}
							if(error.response.data != null){
								toConvert = error.response.data;
							}
							if (toConvert.hasOwnProperty("@class") && toConvert["@class"] == "com/sas/dm/common/model/ErrorResponse") {
								XhrUtils.getInstance().convertJsonObjectsToClass(toConvert, "com/sas/dm/common/model/ErrorResponse").then( _lang.hitch(this, function(converted){
									if(converted.get("errorCode") == 0){
										converted.set("message", "errorNoResponseFromServer_txt");
									}
									//attempt to resolve the message in case it has a valid message key
									converted.resolveMessage(resourceBundle);
									errorHandler(converted);
								}));
							}
							else {
								// Assume this is a standard error response from DMSharedServices midtier.
								// Create an ErrorResponse object from what we have in that response.
								var errResp = new ErrorResponse();
								toConvert = error.response;
								if (toConvert.hasOwnProperty("status")) {
									errResp.set("httpStatusCode", toConvert["status"]);
								}
								if (toConvert.hasOwnProperty("data") && toConvert["data"] != null) {
									errResp.set("message", toConvert["data"]);
								}else if(toConvert.hasOwnProperty("text")){
									errResp.set("message", toConvert["text"]);
								}
								//attempt to resolve the message in case it has a valid message key
								errResp.resolveMessage(resourceBundle);
								errorHandler(errResp);
							}
						}else{
							errorHandler(error);
						}
					})
				);
			},
			
			_quickAsyncCallAsJSON:function(verb, methodName, params, url, successHandler, errorHandler, classOfReturnType){
				this._quickAsyncCall(verb, methodName, params, url, successHandler, errorHandler, "json", classOfReturnType);
			},

			_quickAsyncCallAsMultipartFormData:function(verb, methodName, params, url, successHandler, errorHandler, classOfReturnType){
				this._quickAsyncCall(verb, methodName, params, url, successHandler, errorHandler, SharedProperties.restType_Multipart, classOfReturnType);
			},
			
			_quickDeserialize:function(/*json String*/ serializedObject, /*Function*/ callback){
				XhrUtils.getInstance().convertJsonObjectsToClass(serializedObject).then( _lang.hitch(this, function(converted){
					callback(converted);
				}));
			},
			
			_buildURL:function(root, methodName){
				if(root == null){
					return methodName;
				}
				if(methodName != null && methodName.length > 0 && methodName.charAt(0) == "/"){
					methodName = methodName.substr(1);
				}
				if(root.charAt(root.length-1) == "/" || (methodName != null && methodName.length > 0 && methodName.charAt(0) == "/")){
					return root + methodName;
				}
				return root + "/" + methodName;
			},

			// displays an error dialog for errors not associated with a particular step
			displayError:function(error /*ErrorMessage*/){
				var errorDialog = new ErrorDialog();
				errorDialog.set("error", error);
				errorDialog.show();
			},
			
			constructor:function(){
				this._directiveObserver = new DirectiveObserver({"controller":this});
				this._preferencesProxy = new PreferencesProxy({"preferencesStrategy":new PreferencesServiceStrategy({"preferencesRestUrl":dojoString.substitute(SharedProperties.preferencesServiceURL,[encodeURIComponent(config.sas.commons.application.userDetails.userid)])})});
				this._genericCache = {};
			},
			
			//////////////////////////////////////////////////////////////////////////////////////////////////////////
			/////////////////////////////////////////////////////////////////////////////////////////////////////////
			// TODO: this mockTransformations list will need to be deleted when loading transformations from mid-tier
			
			_mockClenseTransformations:[
									new Step(
											   {
												"name":"transformationFilter_label_$I18N$", 
												"description":"transformationFilterDescription_txt_$I18N$", 
												"iconClass":"Filter",
											    "type":"transform", 
											    "converterBean":"ds2FilterConverter",
									      	"id":"FilterDataTransform", 
									      	"description":"transformationFilterDescription_txt_$I18N$", 
									      	"uiEditorClass":"com/sas/dm/datadirector/views/components/FilterColumns",
									      	"requiredInputs":{
									      						"$columnList":null
									      					 },
									      	"outputMetadata":[
									      	                 { //StepOutputMetadata
									      	                	 "key":"filters",
									      	                	 "type":"Predicate",
									      	                	 "isRequired":false
									      	                 },
									      	                 { //StepOutputMetadata
									      	                	 "key":"allRows",
									      	                	 "type":"boolean",
									      	                	 "isRequired":false
									      	                 },					
									      	                 { //StepOutputMetadata
									      	                	 "key":"filterExpression",
									      	                	 "type":"Expression",
									      	                	 "isRequired":false
									      	                 },											      	                 
		     				            	                 { //StepOutputMetadata
		     				            	                	 "key":"$filtersValid",
		     				            	                	 "type":"boolean",
		     				            	                	 "isRequired":true
		     				            	                 }
									      	                 ],
									      	"uiParams":{
									      		"isTransformation":true
									      	}                 
													
											   }),
										new Step(
												   {
													"name":"transformationGenerateMatchCodes_label_$I18N$", 
													"description":"transformationGenerateMatchCodesDescription_txt_$I18N$", 
													"iconClass":"MatchCodes",
												    "type":"transform", 
												    "converterBean":"matchCodeConverter",
											      	"id":"GenerateMatchCodesTransform", 
											      	"uiEditorClass":"com/sas/dm/datadirector/views/components/GenerateMatchCodes",
											      	"requiredInputs":{
											      						"$columnList":null
											      					 },
											      	"outputMetadata":[
											      	                 { //StepOutputMetadata
											      	                	 "key":"matchCodes",
											      	                	 "type":"array",
											      	                	 "isRequired":true
											      	                 },
											      	                 { //StepOutputMetadata
				     				            	                	 "key":"$matchCodesValid",
				     				            	                	 "type":"boolean",
				     				            	                	 "isRequired":true,
				     				            	                 },
				     				            	                 {
				     				            	                	 "key":"locale",
				     				            	                	 "type":"QKBLocale",
				     				            	                	 "isRequired":true			     				            	                	 
				     				            	                 },
				     				            	                 {
				     				            	                	 "key":"$newColumnList",
				     				            	                	 "type":"array",
				     				            	                	 "mapsTo":"$columnList",
				     				            	                	 "scope":"protected"
				     				            	                 }	
											      	                 ],
											      	"uiParams":{
											      		"isTransformation":true
											      	}                 														
												   }),										   
                                     new Step(
                                               {
                                                "name":"transformationIdentificationAnalysis_label_$I18N$", 
                                                "description":"transformationIdentificationAnalysisDescription_txt_$I18N$", 
                                                "iconClass":"IdentificationAnalysis",
                                                "type":"transform", 
                                                "converterBean":"identificationAnalysisConverter",
                                                "id":"IdentificationAnalysisTransform", 
                                                "uiEditorClass":"com/sas/dm/datadirector/views/components/IdentificationAnalysis",
                                                "requiredInputs":{
                                                                    "$columnList":null
                                                                 },
                                                "outputMetadata":[
                                                                 { //StepOutputMetadata
                                                                     "key":"identificationAnalysis",
                                                                     "type":"array",
                                                                     "isRequired":true
                                                                 },
                                                                 { //StepOutputMetadata
                                                                     "key":"$identificationAnalysisValid",
                                                                     "type":"boolean",
                                                                     "isRequired":true,
                                                                 },
                                                                 {
                                                                     "key":"locale",
                                                                     "type":"QKBLocale",
                                                                     "isRequired":true                                                                   
                                                                 },
                                                                 {
                                                                     "key":"$newColumnList",
                                                                     "type":"array",
                                                                     "mapsTo":"$columnList",
                                                                     "scope":"protected"
                                                                 }
                                                                 ],
                                                "uiParams":{
                                                    "isTransformation":true
                                                }                                                                       
                                               }),
									   new Step(
			                        		   {
			                        			"name":"transformationManageColumns_label_$I18N$", 
			                        			"iconClass":"Columns",
			                        		    "type":"transform",
			                        		    "converterBean":"ds2ColumnSubsetConverter",
			     				            	"id":"ColumnSubsetTransform", 
			     				            	"description":"transformationManageColumnsDescription_txt_$I18N$", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/SubsetColumns",
			     				            	"requiredInputs":{
			     				            	                  "$columnList":null 
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"columns",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList"
			     				            	                 },
			     				            	                 {
			     				            	                	 "key":"columnExpressions",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired": false
			     				            	                 }
			     				            	                 ],
 				            	                "uiParams":{
 			     				            		"isTransformation":true
 			     				            	}
			                        		   }),
									   new Step(
			                        		   {
			                        			"name":"transformationStandardize_label_$I18N$", 
			                        			"description":"transformationStandardizeDescription_txt_$I18N$", 
			                        			"iconClass":"StandardizeData",
			                        		    "type":"transform", 
			                        		    "converterBean":"standardizeDataConverter",
			     				            	"id":"StandardizeDataTransform", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/StandardizeData",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"standards",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"$standardsValid",
			     				            	                	 "type":"boolean",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 {
			     				            	                	"key":"locale",
			     				            	                	"type":"QKBLocale",
			     				            	                	"isRequired":true			     				            	                	 
			     				            	                 },			     				            	                
			     				            	                 {
			     				            	                	 "key":"$newColumnList",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList",
			     				            	                	 "scope":"protected"
			     				            	                 }			     				            	                 			     				            	                 
			     				            	                 ],
			     				            	                 
			     			     				 "uiParams":{
			    			     				           		"isTransformation":true
			    			     				        	}                				                        				
			                        		   }),
									   new Step(
			                        		   {
			                        			"name":"transformationChangeCase_label_$I18N$", 
			                        			"description":"transformationChangeCaseDescription_txt_$I18N$", 
			                        			"iconClass":"ChangeCase",
			                        		    "type":"transform", 
			                        		    "converterBean":"changeCaseConverter",
			     				            	"id":"ChangeCaseTransform", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/ChangeCase",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"caseChanges",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"$caseChangesValid",
			     				            	                	 "type":"boolean",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 {
			     				            	                	"key":"locale",
			     				            	                	"type":"QKBLocale",
			     				            	                	"isRequired":true			     				            	                	 
			     				            	                 },			     				            	                
			     				            	                 {
			     				            	                	 "key":"$newColumnList",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList",
			     				            	                	 "scope":"protected"
			     				            	                 }			     				            	                 			     				            	                 
			     				            	                 ],
			     				            	                 
			     			     				 "uiParams":{
			    			     				           		"isTransformation":true
			    			     				        	}                				                        				
			                        		   }),		
									   new Step(
			                        		   {
			                        			"name":"transformationGenderAnalysis_label_$I18N$", 
			                        			"description":"transformationGenderDescription_txt_$I18N$", 
			                        			"iconClass":"AnalyzeGender",
			                        		    "type":"transform", 
			                        		    "converterBean":"genderAnalysisConverter",
			     				            	"id":"GenderAnalysisTransform", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/GenderAnalysis",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"analyzedGenders",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"$analyzedGendersValid",
			     				            	                	 "type":"boolean",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 {
			     				            	                	"key":"locale",
			     				            	                	"type":"QKBLocale",
			     				            	                	"isRequired":true			     				            	                	 
			     				            	                 },			     				            	                
			     				            	                 {
			     				            	                	 "key":"$newColumnList",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList",
			     				            	                	 "scope":"protected"
			     				            	                 }			     				            	                 			     				            	                 
			     				            	                 ],
			     				            	                 
			     			     				 "uiParams":{
			    			     				           		"isTransformation":true
			    			     				        	}                				                        				
			                        		   }),				
									   new Step(
			                        		   {
			                        			"name":"transformationPatternAnalysis_label_$I18N$", 
			                        			"description":"transformationPatternDescription_txt_$I18N$", 
			                        			"iconClass":"PatternAnalysis",
			                        		    "type":"transform", 
			                        		    "converterBean":"patternAnalysisConverter",
			     				            	"id":"PatternAnalysisTransform", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/PatternAnalysis",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"analyzedPatterns",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"$analyzedPatternsValid",
			     				            	                	 "type":"boolean",
			     				            	                	 "isRequired":true
			     				            	                 },
			     				            	                 {
			     				            	                	"key":"locale",
			     				            	                	"type":"QKBLocale",
			     				            	                	"isRequired":true			     				            	                	 
			     				            	                 },			     				            	                
			     				            	                 {
			     				            	                	 "key":"$newColumnList",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList",
			     				            	                	 "scope":"protected"
			     				            	                 }			     				            	                 			     				            	                 
			     				            	                 ],
			     				            	                 
			     			     				 "uiParams":{
			    			     				           		"isTransformation":true
			    			     				        	}                				                        				
			                        		   }),				                        		   
			                           new Step(
					                        		   {
					                        			"name":"transformationSummarize_label_$I18N$", 
					                        			"description":"transformationSummarizeDescription_txt_$I18N$", 
					                        			"iconClass":"SummarizeRows",
					                        		    "type":"transform", 
					                        		    "converterBean":"ds2ColumnAggregationConverter",
					     				            	"id":"SummarizeRowsTransform", 
					     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/SummarizeRows",
					     				            	"requiredInputs":{
					     				            						"$columnList":null
					     				            					 },
					     				            	"outputMetadata":[
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"aggregations",
					     				            	                	 "type":"array",
					     				            	                	 "isRequired":false,
					     				            	                 },
					     				            	                 {
					     				            	                	"key":"$aggregations",
					     				            	                	//"type":"array",
					     				            	                	"isRequired":false
					     				            	                	//"scope":"internal"
					     				            	                 },
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"groupByList",
					     				            	                	 "type":"array"
					     				            	                 },
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"$valid",
					     				            	                	 "type":"boolean",
					     				            	                	 "isRequired":true,
					     				            	                 },
					     				            	                 {
					     				            	                	 "key":"$newColumnList",
					     				            	                	 "type":"array",
					     				            	                	 "mapsTo":"$columnList",
					     				            	                	 "scope":"protected"
					     				            	                 }
					     				            	                 ],
					     			     				 "uiParams":{
					    			     				           		"isTransformation":true
					    			     				        	}                				                        				
					                        		   }),
									   new Step(
											   {
												"name":"transformationParse_label_$I18N$", 
												"description":"transformationParseDescription_txt_$I18N$", 
												"iconClass":"Parsing",
											    "type":"transform", 
											    "converterBean":"parseColumnsConverter",
										      	"id":"ParseDataTransform", 
										      	"uiEditorClass":"com/sas/dm/datadirector/views/components/ParseData",
										      	"requiredInputs":{
										      						"$columnList":null
										      					 },
										      	"outputMetadata":[
										      	                 { //StepOutputMetadata
										      	                	 "key":"parsedColumns",
										      	                	 "type":"array"
										      	                 },
			     				            	                 {
			     				            	                	 "key":"locale",
			     				            	                	 "type":"QKBLocale",
			     				            	                	 "isRequired":true			     				            	                	 
			     				            	                 },
			     				            	                 {
			     				            	                	 "key":"$newColumnList",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList",
			     				            	                	 "scope":"protected"
			     				            	                 }	
										      	                 ],
										      	"uiParams":{
										      		"isTransformation":true
										      	}                 														
											   }),							                        		   
									   new Step(
			                        		   {
			                        			"name":"transformationFieldExtract_label_$I18N$", 
			                        			"description":"transformationFieldExtracDescription_txt_$I18N$", 
			                        			"iconClass":"FieldExtraction",
			                        		    "type":"transform", 
			                        		    "converterBean":"fieldExtractionConverter",
			     				            	"id":"FieldExtractionTransform", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/FieldExtraction",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
 										      	"outputMetadata":[
 										      	                 { //StepOutputMetadata
 										      	                	 "key":"fieldExtractionColumns",
 										      	                	 "type":"array"
 										      	                 },
 			     				            	                 {
 			     				            	                	 "key":"locale",
 			     				            	                	 "type":"QKBLocale",
 			     				            	                	 "isRequired":true			     				            	                	 
 			     				            	                 },
 			     				            	                 {
 			     				            	                	 "key":"$newColumnList",
 			     				            	                	 "type":"array",
 			     				            	                	 "mapsTo":"$columnList",
 			     				            	                	 "scope":"protected"
 			     				            	                 }	
 										      	                 ],	     				            	                 
			     			     				 "uiParams":{
			    			     				           		"isTransformation":true
			    			     				        	}                				                        				
			                        		   })						                        		   
									/* Commenting out Sort transformation for now
							                   new Step(
									                        {
									                        	"name":"transformationSort_label_$I18N$", 
									                        	"description":"transformationSortDescription_txt_$I18N$", 
									                        	"iconClass":"SortAndManageDups",
									                        	"type":"transform", 
									                        	"converterBean":"sortAndManageDupsDataStepConverter",
									                        	"id":"SortAndManageDupsTransform", 
									                        	"uiEditorClass":"com/sas/dm/datadirector/views/components/SortAndManageDups",
									     				    	"requiredInputs":{
									     				    		"$columnList":null
									     				    	},
									     				    	"outputMetadata":[
									     				    	                  { //StepOutputMetadata
									     				            	                	 "key":"sortAndManageDups",
									     				            	                	 "type":"SortAndManageDups",
									     				       	                	 "isRequired":true,
									     				            	           }
									     				            	           ],
									     				        "uiParams":{
									     				        	"isTransformation":true
									    			     		}                				                        				
									          }), */
					                           
			],
			_mockTransformations:[
			                           new Step(
			                        		   {
			                        			"name":"transformationFilter_label_$I18N$", 
			                        			"description":"transformationFilterDescription_txt_$I18N$", 
			                        			"iconClass":"Filter",
			                        		    "type":"transform", 
			                        		    "converterBean":"ds2FilterConverter",
			     				            	"id":"FilterDataTransform", 
			     				            	"description":"transformationFilterDescription_txt_$I18N$", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/FilterColumns",
			     				            	"requiredInputs":{
			     				            						"$columnList":null
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"filters",
			     				            	                	 "type":"Predicate",
			     				            	                	 "isRequired":false
			     				            	                 },
										      	                 { //StepOutputMetadata
										      	                	 "key":"allRows",
										      	                	 "type":"boolean",
			     				            	                	 "isRequired":false
										      	                 },					
										      	                 { //StepOutputMetadata
										      	                	 "key":"filterExpression",
										      	                	 "type":"Expression",
				     				            	                 "isRequired":false

										      	                 },						     				            	                 
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"$filtersValid",
			     				            	                	 "type":"boolean",
			     				            	                	 "isRequired":true
			     				            	                 }
			     				            	                 ],
			     				            	"uiParams":{
			     				            		"isTransformation":true
			     				            	}                 
			                        				
			                        		   }),
			                           new Step(
					                        		   {
					                        			"name":"transformationSummarize_label_$I18N$", 
					                        			"description":"transformationSummarizeDescription_txt_$I18N$", 
					                        			"iconClass":"SummarizeRows",
					                        		    "type":"transform", 
					                        		    "converterBean":"ds2ColumnAggregationConverter",
					     				            	"id":"SummarizeRowsTransform", 
					     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/SummarizeRows",
					     				            	"requiredInputs":{
					     				            						"$columnList":null
					     				            					 },
					     				            	"outputMetadata":[
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"aggregations",
					     				            	                	 "type":"array",
					     				            	                	 "isRequired":false,
					     				            	                 },
					     				            	                 {
					     				            	                	"key":"$aggregations",
					     				            	                	//"type":"array",
					     				            	                	"isRequired":false
					     				            	                	//"scope":"internal"
					     				            	                 },
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"groupByList",
					     				            	                	 "type":"array"
					     				            	                 },
					     				            	                 { //StepOutputMetadata
					     				            	                	 "key":"$valid",
					     				            	                	 "type":"boolean",
					     				            	                	 "isRequired":true,
					     				            	                 },
					     				            	                 {
					     				            	                	 "key":"$newColumnList",
					     				            	                	 "type":"array",
					     				            	                	 "mapsTo":"$columnList",
					     				            	                	 "scope":"protected"
					     				            	                 }
					     				            	                 ],
					     			     				 "uiParams":{
					    			     				           		"isTransformation":true
					    			     				        	}                				                        				
					                        		   }),			                        		   
			                           new Step(
			                        		   {
			                        			"name":"transformationManageColumns_label_$I18N$", 
			                        			"iconClass":"Columns",
			                        		    "type":"transform",
			                        		    "converterBean":"ds2ColumnSubsetConverter",
			     				            	"id":"ColumnSubsetTransform", 
			     				            	"description":"transformationManageColumnsDescription_txt_$I18N$", 
			     				            	"uiEditorClass":"com/sas/dm/datadirector/views/components/SubsetColumns",
			     				            	"requiredInputs":{
			     				            	                  "$columnList":null 
			     				            					 },
			     				            	"outputMetadata":[
			     				            	                 { //StepOutputMetadata
			     				            	                	 "key":"columns",
			     				            	                	 "type":"array",
			     				            	                	 "mapsTo":"$columnList"
			     				            	                 },
			     				            	                 {
			     				            	                	 "key":"columnExpressions",
			     				            	                	 "type":"array",
			     				            	                	 "isRequired": false
			     				            	                 }
			     				            	                 ],
 				            	                "uiParams":{
 			     				            		"isTransformation":true
 			     				            	}
			                        		   })
			                           /* Commenting out Sort transformation for now
			                        new Step(
			                        		{
			                        			"name":"transformationSort_label_$I18N$", 
			                        			"description":"transformationSortDescription_txt_$I18N$", 
			                        			"iconClass":"SortAndManageDups",
			                        			"type":"transform", 
			                        			"converterBean":"sortAndManageDupsDataStepConverter",
			                        			"id":"SortAndManageDupsTransform", 
			                        			"uiEditorClass":"com/sas/dm/datadirector/views/components/SortAndManageDups",
			                        			"requiredInputs":{
			                        				"$columnList":null
			                        			},
			                        			"outputMetadata":[
			                        			                  { //StepOutputMetadata
			                        			                	  "key":"sortAndManageDups",
			                        			                	  "type":"SortAndManageDups",
			                        			                	  "isRequired":true,
			                        			                  }
			                        			                  ],
			                        			"uiParams":{
			                        				"isTransformation":true
			                        			}                				                        				
			                        		}), */
			                           ]
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			
			
		});
		return DataDirectorController;
});
