/**
 *  This is the presentation model used to along with components/DatabaseConfigPane.js
 *  and markup/DatabaseConfigPane.html.  DatabaseConfigPane.js binds this model to the DatabaseConfigPane markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (DatabaseConfigPane.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "com/sas/dm/common/util/StringUtils",
        "dojo/Stateful",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, _ContextAwareMixin, StringUtils, Stateful, resources) {
	
	// Subclass Stateful so the component class can bind to the getters and setters defined here
	// Subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	var DatabaseConfigPanePM = declare([Stateful, _ContextAwareMixin], {
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter:function(value/*DataDirectorController*/) {
			this._controller = value;
			this._loadServers();
		},
		_controllerGetter:function() {
			return this._controller;
		},
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context) {
			// Grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
		_jdbcServers:null,
		_jdbcServersGetter:function() {
			return this._jdbcServers;
		},
		_jdbcServersSetter:function(value /*Array of JdbcServer*/) {
			if (this._jdbcServers == null) {
				this._jdbcServers = [];
			}
			else {
				while (this._jdbcServers.length > 0) {
					this._jdbcServers.pop();
				}
			}
			
			if (value != null) {
				for (var i = 0; i < value.length; i++) {
					this._jdbcServers.push(value[i]);
				}
			}
		},
		
		_jdbcServersForSubmit:null,
		_jdbcServersForSubmitGetter:function() {
			return this._jdbcServersForSubmit;
		},
		_jdbcServersForSubmitSetter:function(value /*Array of JdbcServer*/) {
			this._jdbcServersForSubmit = value;
		},
		
		_errorTitle:null,
		_errorTitleGetter:function() {
			return this._errorTitle;
		},
		_errorTitleSetter:function(value/*String*/) {
			this._errorTitle = value;
		},
		
		_errorDetails:null,
		_errorDetailsGetter:function() {
			return this._errorDetails;
		},
		_errorDetailsSetter:function(value/*String*/) {
			this._errorDetails = value;
		},
		
		_loadServers:function() {
			if (this._controller == null) {
				return;
			}
			
			this.set("isBusy", true);			
			this._controller.getJDBCConnections(_lang.hitch(this, this._onLoadServersSuccess), _lang.hitch(this, this._onLoadServersError));
		},
		
		_onLoadServersSuccess:function(data) {
			this.set("jdbcServers", data);
			this.set("isBusy", false);
		},
		
		_onLoadServersError:function(error) {
			console.log(error);
			this._generateErrorDetails(error);
			this.set("isBusy", false);
		},
		
		_generateErrorDetails:function(error/*Object*/) {
			if (error == null) {
				this.set("errorTitle", null);
                this.set("errorDetails", null);
                return;
			}
			var resolvedMessage = error.resolveMessage(resources);
            var errorDetails = error.get("details");
		    var errorMessage = "";
		    if(errorDetails != null){
		        if(errorDetails instanceof Array){
		            var errorDetailsLength = errorDetails.length;
		            for(var i=0; i<errorDetailsLength; i++){
		                var errorDetail = errorDetails[i];
		                if(errorDetail == null){
		                    continue;
		                }
		                var delim = "";
		                if(errorMessage.length > 0){
		                    delim = "\n";
		                }
		                errorMessage = errorMessage + delim + errorDetail;
		            }
		        }else{
		            errorMessage = errorDetails;
		        }
		    }
		    
		    if (StringUtils.isEmpty(errorMessage) && StringUtils.isEmpty(resolvedMessage)) {
		        // No error message or details - default title, default details
	            this.set("errorTitle", resources.defaultErrorDialog_title); 
	            this.set("errorDetails", resources.defaultErrorDialog_message);
		    }
		    else if (StringUtils.isEmpty(errorMessage)) {
		        // No error message - default title, details = details
		        this.set("errorTitle", resources.defaultErrorDialog_title);
		        this.set("errorDetails", resolvedMessage);
            }
		    else if (StringUtils.isEmpty(resolvedMessage)) {
		        // No details - default title, details = error message
		        this.set("errorTitle", resources.defaultErrorDialog_title);
	            this.set("errorDetails", errorMessage);
		    }
		    else {
		        // title = error message, details = details
                this.set("errorTitle", resolvedMessage);
                this.set("errorDetails", errorMessage);
		    }
		},
		
		handleSubmit:function(successHandler, errorHandler) {
			this._pendingRestCalls = 0;
			this._finalSuccessHander = successHandler;
			this._finalErrorHandler = errorHandler;
			this._anySubmitErrors = false;
			this._generateErrorDetails(null);
			
			var anyChanges = false;
			var servers = this.get("jdbcServersForSubmit");
			for (var i = 0; i < servers.length; i++) {
				var s = servers[i];
				var isNew = s.get("$isNew");
				var isModified = s.get("$isModified");
				var isDeleted = s.get("$isDeleted");
				if (isDeleted) {
					if (!isNew) {
						anyChanges = true;
						this._pendingRestCalls++;
						this._controller.deleteJDBCConnection(s, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
					}
				}
				else if (isNew || isModified) {
					// Convert password to base64
					var password = s.get("$password");
					var domain = s.get("authDomain");
					var pwBytes = [];
					for (var j = 0; j < password.length; j++) {
						pwBytes.push(password.charCodeAt(j));
					}
					var password_base64 = window.btoa(pwBytes);
					
					anyChanges = true;
					this._pendingRestCalls += 2;
					this._controller.setJDBCConnection(s, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
					this._controller.setCredentials(s.get("$userName"), password_base64, domain, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
				}
			}
			
			if (anyChanges === false) {
				successHandler.call(this, {});
			}
		},
		
		_finalSuccessHander:null,
		_finalErrorHandler:null,
		_pendingRestCalls:0,
		_anySubmitErrors:false,
		
		_controllerSuccessHandler:function(result) {
			this._pendingRestCalls--;
			
			//need to check the result to see if it is a Database so we can mark it as no longer new
			if (result != null && result != "") {
				var id = result.get("id");
				var servers = this.get("jdbcServersForSubmit");
				for (var i = 0; i < servers.length; i++) {
					var s = servers[i];
					if (id == s.get("id")) {
						s.set("$isNew", false);
						s.set("$isModified", false);
						break;
					}
				}
			}
			if (this._pendingRestCalls == 0 && !this._anySubmitErrors) {
				this._finalSuccessHander(result);
			}
		},
		
		_controllerErrorHandler:function(error) {
			this._pendingRestCalls--;
			this._anySubmitErrors = true;
			this._generateErrorDetails(error);
			if (this._pendingRestCalls == 0) {
				this._finalErrorHandler(error);
			}
		},
		
		destroy:function() {
			this.inherited(arguments);
		},
		
		constructor:function() {
			
		}
	});
	return DatabaseConfigPanePM;
});
