/**
 *  This is the presentation model used to along with components/LASRServerConfigPane.js
 *  and markup/LASRServerConfigPane.html.  LASRServerConfigPane.js binds this model to the LASRServerConfigPane markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (LASRServerConfigPane.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/datadirector/models/servers/HadoopServer",
        "dojo/Stateful",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, _ContextAwareMixin, StringUtils, HadoopServer, Stateful, resources) {
	
	// Subclass Stateful so the component class can bind to the getters and setters defined here
	// Subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	var LASRServerConfigPanePM = declare([Stateful, _ContextAwareMixin], {
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter:function(value/*DataDirectorController*/) {
			this._controller = value;
			this._loadLasrServers();
		},
		_controllerGetter:function() {
			return this._controller;
		},
		
		_isBusy:false,
        _isBusyGetter:function(){
            return this._isBusy;
        },
        _isBusySetter:function(/*Boolean*/ value){
            this._isBusy = value;
        },
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context) {
			// Grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
		_lasrServers:null,
		_lasrServersGetter:function() {
			return this._lasrServers;
		},
		_lasrServersSetter:function(value /*Array of LASRServer*/) {
			if (this._lasrServers == null) {
				this._lasrServers = [];
			}
			else {
				while (this._lasrServers.length > 0) {
					this._lasrServers.pop();
				}
			}
			
			if (value != null) {
				for (var i = 0; i < value.length; i++) {
					this._lasrServers.push(value[i]);
				}
			}
		},
		
		_$lasrServersForSubmit:null,
		_$lasrServersForSubmitGetter:function() {
			return this._$lasrServersForSubmit;
		},
		_$lasrServersForSubmitSetter:function(value /*Array of LASRServer*/) {
			this._$lasrServersForSubmit = value;
		},
		
		_errorTitle:null,
		_errorTitleGetter:function() {
			return this._errorTitle;
		},
		_errorTitleSetter:function(value/*String*/) {
			this._errorTitle = value;
		},
		
		_errorDetails:null,
		_errorDetailsGetter:function() {
			return this._errorDetails;
		},
		_errorDetailsSetter:function(value/*String*/) {
			this._errorDetails = value;
		},
		
		_loadLasrServers:function() {
			if (this._controller == null) {
				return;
			}
			this.set("isBusy", true);
			this._controller.getLasrServers(_lang.hitch(this, this._onLoadLasrServersSuccess), _lang.hitch(this, this._onLoadLasrServersError));
		},
		
		_onLoadLasrServersSuccess:function(data) {
			this.set("lasrServers", data);
			this.set("isBusy", false);
		},
		
		_onLoadLasrServersError:function(error) {
			console.log(error);
			this._generateErrorDetails(error);
			this.set("isBusy", false);
		},
		
		_generateErrorDetails:function(error/*Object*/) {
			if (error == null) {
				this.set("errorTitle", null);
                this.set("errorDetails", null);
                return;
			}
			var resolvedMessage = error.resolveMessage(resources);
            var errorDetails = error.get("details");
		    var errorMessage = "";
		    if(errorDetails != null){
		        if(errorDetails instanceof Array){
		            var errorDetailsLength = errorDetails.length;
		            for(var i=0; i<errorDetailsLength; i++){
		                var errorDetail = errorDetails[i];
		                if(errorDetail == null){
		                    continue;
		                }
		                var delim = "";
		                if(errorMessage.length > 0){
		                    delim = "\n";
		                }
		                errorMessage = errorMessage + delim + errorDetail;
		            }
		        }else{
		            errorMessage = errorDetails;
		        }
		    }
		    
		    if (StringUtils.isEmpty(errorMessage) && StringUtils.isEmpty(resolvedMessage)) {
		        // No error message or details - default title, default details
	            this.set("errorTitle", resources.defaultErrorDialog_title); 
	            this.set("errorDetails", resources.defaultErrorDialog_message);
		    }
		    else if (StringUtils.isEmpty(errorMessage)) {
		        // No error message - default title, details = details
		        this.set("errorTitle", resources.defaultErrorDialog_title);
		        this.set("errorDetails", resolvedMessage);
            }
		    else if (StringUtils.isEmpty(resolvedMessage)) {
		        // No details - default title, details = error message
		        this.set("errorTitle", resources.defaultErrorDialog_title);
	            this.set("errorDetails", errorMessage);
		    }
		    else {
		        // title = error message, details = details
                this.set("errorTitle", resolvedMessage);
                this.set("errorDetails", errorMessage);
		    }
		},
		
		handleSubmit:function(successHandler, errorHandler) {
			this._pendingRestCalls = 0;
			this._finalSuccessHander = successHandler;
			this._finalErrorHandler = errorHandler;
			this._anySubmitErrors = false;
			this._generateErrorDetails(null);
			
			var anyChanges = false;
			var lasrServers = this.get("$lasrServersForSubmit");
			for (var i = 0; i < lasrServers.length; i++) {
				var s = lasrServers[i];
				var isNew = s.get("$isNew");
				var isModified = s.get("$isModified");
				var isDeleted = s.get("$isDeleted");
				if (isDeleted) {
					if (!isNew) {
						anyChanges = true;
						this._pendingRestCalls++;
						this._controller.deleteLasrServer(s, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
					}
				}
				else if (isNew || isModified) {
					//need to set some defaults here
					if (isNew) {
						s.set("id", this._createGuid());
						s.set("paths", []);
					}
					
					//convert password to base64
					var mdPassword = s.get("$metadataPassword");
					var mdDomain = s.get("authDomain");
					var mdBytes = [];
					for (var j = 0; j < mdPassword.length; j++) {
						mdBytes.push(mdPassword.charCodeAt(j));
					}
					var md_password_base64 = window.btoa(mdBytes);
					
					anyChanges = true;
					this._pendingRestCalls += 2;
					this._controller.setLasrServer(s, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
					this._controller.setCredentials(s.get("$metadataUser"), md_password_base64, mdDomain, _lang.hitch(this, this._controllerSuccessHandler), _lang.hitch(this, this._controllerErrorHandler));
				}
			}
			
			if (anyChanges === false) {
				successHandler.call(this, {});
			}
		},
		
		_finalSuccessHander:null,
		_finalErrorHandler:null,
		_pendingRestCalls:0,
		_anySubmitErrors:false,
		
		_controllerSuccessHandler:function(result) {
			this._pendingRestCalls--;
			
			//need to check the result to see if it is a LASR server so we can mark it as no longer new
			if (result != null && result != "") {
				var id = result.get("id");
				var servers = this.get("$lasrServersForSubmit");
				for (var i = 0; i < servers.length; i++) {
					var server = servers[i];
					if (id == server.get("id")) {
						server.set("$isNew", false);
						server.set("$isModified", false);
						break;
					}
				}
			}
			if (this._pendingRestCalls == 0 && !this._anySubmitErrors) {
				this._finalSuccessHander(result);
			}
		},
		
		_controllerErrorHandler:function(error) {
			this._pendingRestCalls--;
			this._anySubmitErrors = true;
			this._generateErrorDetails(error);
			if (this._pendingRestCalls == 0) {
				this._finalErrorHandler(error);
			}
		},
		
		_createGuid:function() {
			return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			    var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
			    return v.toString(16);
			});
		},
		
		destroy:function() {
			this.inherited(arguments);
		},
		
		constructor:function() {
			
		}
	});
	return LASRServerConfigPanePM;
});