/**
 *  This is the presentation model used to along with components/ConfigurationDialog.js
 *  and markup/ConfigurationDialog.html.  ConfigurationDialog.js binds this model to the ConfigurationDialog markup.
 *  A reference to the master controller can be acquired through the context.  The context is set on this instance
 *  from the Binder component class (ConfigurationDialog.js).  Subclassing _ContextAwareMixin ensures the  reference 
 *  to the context is stored and the _onSetContext method is called when the context becomes available.
 */

define(["dojo/_base/declare",
        "dojo/_base/lang",
        "com/sas/dm/common/components/base/_ContextAwareMixin",
        "com/sas/dm/common/util/StringUtils",
        "com/sas/dm/datadirector/util/SharedProperties",
        "com/sas/dm/datadirector/models/servers/LASRServer",
        "dojo/Stateful",
        "dojo/i18n!com/sas/dm/datadirector/nls/DataDirector"], function(declare, _lang, _ContextAwareMixin, StringUtils, SharedProperties, LASRServer, Stateful, resources) {
	
	// Subclass Stateful so the component class can bind to the getters and setters defined here
	// Subclass _ContextAwareMixin so the component class can set a context and the _onSetContext method is called
	var LASRServerConfigurationDialogPM = declare([Stateful, _ContextAwareMixin], {
		
		/*DataDirectorController*/
		_controller:null,
		_controllerSetter:function(value/*DataDirectorController*/) {
			this._controller = value;
			this._loadMetadataCredentials();
		},
		_controllerGetter:function() {
			return this._controller;
		},
		
		// Override - from _ContextAwareMixin - this method is called when a context is available
		_onSetContext:function(context) {
			// Grab a reference to the controller from the context
			this.set("controller", context.get("Controller"));
		},
		
		_lasrServer:null,
		_lasrServerGetter:function() {
			return this._lasrServer;
		},
		_lasrServerSetter:function(value /*LASRServer*/) {
			if (value != null) {
				var user = value.get("sshUserId");
				if (user == null) {
					value.set("sshUserId", SharedProperties.lasrConfiguration_default_user);  // Assume 2.2 or older server definition.  Add the default user ID.
				}
			}
			this._lasrServer = value;
		},
		
		_metadataUser:null,
		_metadataUserGetter:function() {
			return this._metadataUser;
		},
		_metadataUserSetter:function(value /*String*/) {
			this._metadataUser = value;
		},
		
		_metadataPassword:null,
		_metadataPasswordGetter:function() {
			return this._metadataPassword;
		},
		_metadataPasswordSetter:function(value /*String*/) {
			this._metadataPassword = value;
		},
		
		_errorTitle:null,
		_errorTitleGetter:function() {
			return this._errorTitle;
		},
		_errorTitleSetter:function(value/*String*/) {
			this._errorTitle = value;
		},
		
		_errorDetails:null,
		_errorDetailsGetter:function() {
			return this._errorDetails;
		},
		_errorDetailsSetter:function(value/*String*/) {
			this._errorDetails = value;
		},
		
		_generateErrorDetails:function(error/*Object*/) {
			if (error == null) {
				this.set("errorTitle", null);
                this.set("errorDetails", null);
                return;
			}
			var resolvedMessage = error.resolveMessage(resources);
            var errorDetails = error.get("details");
		    var errorMessage = "";
		    if(errorDetails != null){
		        if(errorDetails instanceof Array){
		            var errorDetailsLength = errorDetails.length;
		            for(var i=0; i<errorDetailsLength; i++){
		                var errorDetail = errorDetails[i];
		                if(errorDetail == null){
		                    continue;
		                }
		                var delim = "";
		                if(errorMessage.length > 0){
		                    delim = "\n";
		                }
		                errorMessage = errorMessage + delim + errorDetail;
		            }
		        }else{
		            errorMessage = errorDetails;
		        }
		    }
		    
		    if (StringUtils.isEmpty(errorMessage) && StringUtils.isEmpty(resolvedMessage)) {
		        // No error message or details - default title, default details
	            this.set("errorTitle", resources.defaultErrorDialog_title); 
	            this.set("errorDetails", resources.defaultErrorDialog_message);
		    }
		    else if (StringUtils.isEmpty(errorMessage)) {
		        // No error message - default title, details = details
		        this.set("errorTitle", resources.defaultErrorDialog_title);
		        this.set("errorDetails", resolvedMessage);
            }
		    else if (StringUtils.isEmpty(resolvedMessage)) {
		        // No details - default title, details = error message
		        this.set("errorTitle", resources.defaultErrorDialog_title);
	            this.set("errorDetails", errorMessage);
		    }
		    else {
		        // title = error message, details = details
                this.set("errorTitle", resolvedMessage);
                this.set("errorDetails", errorMessage);
		    }
		},
		
		deleteCredentials:function() {
			if (this._controller == null) {
				return;
			}
			var domain = this._lasrServer.get("authDomain");
			this.set("isBusyQueue", true);
			this._controller.deleteCredentials(domain, _lang.hitch(this, this._onDeleteCredentialsSuccess), this._generateErrorHandler("configurationDialog_deleteHadoopCredentials_error_txt", _lang.hitch(this, this._onDeleteCredentialsError)));
		},
		
		_onDeleteCredentialsSuccess:function(data) {
			this._lasrServer.set("$isCredentialsLoaded",false);
			this._loadMetadataCredentials();
			this.set("isBusyQueue", false);	
		},
		
		_onDeleteCredentialsError:function(error) {
			console.error(error);
			this._generateErrorDetails(error);
			this.set("isBusyQueue", false);			
		},
		
		_loadMetadataCredentials:function() {
			if (this._lasrServer != null) {
				if (this._lasrServer.get("$isCredentialsLoaded") == true) {
					this.set("metadataUser", this._lasrServer.get("$metadataUser"));
					this.set("metadataPassword", this._lasrServer.get("$metadataPassword"));
				}
				else {
					var authDomain = this._lasrServer.get("authDomain");
					if (authDomain != null && authDomain.length > 0) {
						this._controller.getCredentials(authDomain, _lang.hitch(this, this._onLoadCredentialsSuccess), _lang.hitch(this, this._onLoadCredentialsError));
					}
				}
			}
		},
		
		_onLoadCredentialsSuccess:function(data) {
			if (data != null && data != "") {
				this.set("metadataUser", data.get("principal"));
				this.set("metadataPassword", data.get("credential"));
			}
			if (this._lasrServer != null) {
				if (data && data != "") {
					this._lasrServer.set("$metadataUser", data.get("principal"));
					this._lasrServer.set("$metadataPassword", data.get("credential"));
					
					this.set("metadataUser", data.get("principal"));
					this.set("metadataPassword", data.get("credential"));
				}
				else
				{
					this._lasrServer.set("$metadataUser", "");
					this._lasrServer.set("$metadataPassword", "");
					
					//set to null to guarantee that set fires update event for ui watchers
					this._metadataUser = null;
					this.set("metadataUser", "");
					//set to null to guarantee that set fires update event for ui watchers
					this._metadataPassword = null;
					this.set("metadataPassword", "");
				}
				this._lasrServer.set("$isCredentialsLoaded", true);
			}
		},
		
		_onLoadCredentialsError:function(error) {
			console.log(error);
			this._generateErrorDetails(error);
			if (this._lasrServer != null) {
				this._lasrServer.set("$metadataUser", "");
				this._lasrServer.set("$metadataPassword", "");
				this._lasrServer.set("$isCredentialsLoaded", true);
			}
		},
		
		_generateErrorHandler: function(/*String (resource key)*/defaultErrorMessageKey, /*Function*/ errorHandler){
			return _lang.hitch(this, function(/*ErrorResponse*/error){
				if(error != null){
					if(StringUtils.isEmpty(error.resolveMessage(resources))){
						error.set("$isResolved", false);
						error.set("message", defaultErrorMessageKey);
					} 
				}
				errorHandler(error);
			});
		},
		
		handleSubmit:function(server, successHandler, errorHandler) {
			if (this._lasrServer == null) {
				server.set("$metadataUser", this._metadataUser);
				server.set("$metadataPassword", this._metadataPassword);
				server.set("$isCredentialsLoaded", true);
				this.set("lasrServer", server);
			}
			else {
				var s = this._lasrServer;
				s.set("name", server.get("name"));
				s.set("description", server.get("description"));
				s.set("hostName", server.get("hostName"));
				s.set("port", server.get("port"));
				s.set("sshUserId", server.get("sshUserId"));
				s.set("signerServicePath", server.get("signerServicePath"));
				s.set("useSasIola", server.get("useSasIola"));
				s.set("defaultTag", server.get("defaultTag"));
				s.set("metadataServer", server.get("metadataServer"));
				s.set("metadataPort", server.get("metadataPort"));
				s.set("metadataRepository", server.get("metadataRepository"));
				s.set("folderForNewTables", server.get("folderForNewTables"));
				s.set("folderForLibraryOfTables", server.get("folderForLibraryOfTables"));
				s.set("defaultPort", server.get("defaultPort"));
				s.set("$metadataUser", this._metadataUser);
				s.set("$metadataPassword", this._metadataPassword);
				s.set("$isCredentialsLoaded", true);
			}
			
			successHandler("success");
		},
		
		destroy:function() {
			this.inherited(arguments);
		},
		
		constructor:function() {
			
		}
	});
	return LASRServerConfigurationDialogPM;
});